#!/usr/bin/env python3
# encoding: utf-8

from __future__ import annotations

__all__ = ["check_response", "P123OpenClient", "P123Client"]

from collections.abc import (
    AsyncIterable, Awaitable, Buffer, Callable, Coroutine, 
    ItemsView, Iterable, Iterator, Mapping, MutableMapping, 
)
from errno import EIO, EISDIR, ENOENT
from functools import partial
from hashlib import md5
from http.cookiejar import CookieJar
from inspect import isawaitable
from itertools import chain
from os import fsdecode, fstat, PathLike
from os.path import basename
from re import compile as re_compile
from tempfile import TemporaryFile
from typing import cast, overload, Any, Literal
from uuid import uuid4
from warnings import warn

from aiofile import async_open
from asynctools import ensure_async
from filewrap import (
    bio_chunk_iter, bio_chunk_async_iter, buffer_length, 
    bytes_iter_to_reader, bytes_iter_to_async_reader, 
    copyfileobj, copyfileobj_async, SupportsRead, 
)
from hashtools import file_digest, file_digest_async
from http_request import encode_multipart_data, encode_multipart_data_async, SupportsGeturl
from iterutils import run_gen_step
from property import locked_cacheproperty
from yarl import URL

from .exception import P123OSError, P123BrokenUpload


# 默认使用的域名
DEFAULT_BASE_URL = "https://www.123pan.com/b"
DEFAULT_LOGIN_BASE_URL = "https://login.123pan.com"
DEFAULT_OPEN_BASE_URL = "https://open-api.123pan.com"
# 默认的请求函数
_httpx_request = None


def get_default_request():
    global _httpx_request
    if _httpx_request is None:
        from httpx_request import request
        _httpx_request = partial(request, timeout=(5, 60, 60, 5))
    return _httpx_request


def default_parse(_, content: Buffer, /):
    from orjson import loads
    if isinstance(content, (bytes, bytearray, memoryview)):
        return loads(content)
    else:
        return loads(memoryview(content))


def complete_url(
    path: str, 
    base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
) -> str:
    if path.startswith("//"):
        return "https:" + path
    elif path.startswith(("http://", "https://")):
        return path
    if not base_url:
        base_url = DEFAULT_BASE_URL
    elif callable(base_url):
        base_url = base_url()
    if not path.startswith("/"):
        path = "/api/" + path
    return base_url + path


def dict_to_lower[K, V](
    d: Mapping[K, V] | Iterable[tuple[K, V]], 
    /, 
    *ds: Mapping[K, V] | Iterable[tuple[K, V]], 
    **kwd, 
) -> dict[K, V]:
    return {
        (k.lower() if isinstance(k, str) else k): v # type: ignore
        for k, v in cast(Iterator[tuple[K, V]], chain(items(d), *map(items, ds), kwd.items()))
    }


def dict_to_lower_merge[K, V](
    d: Mapping[K, V] | Iterable[tuple[K, V]], 
    /, 
    *ds: Mapping[K, V] | Iterable[tuple[K, V]], 
    **kwd, 
) -> dict[K, V]:
    m: dict[K, V] = {}
    setdefault = m.setdefault
    for k, v in cast(Iterator[tuple[K, V]], chain(items(d), *map(items, ds), kwd.items())):
        if isinstance(k, str):
            k = k.lower() # type: ignore
        setdefault(k, v)
    return m


def update_headers_in_kwargs(
    request_kwargs: dict, 
    /, 
    *args, 
    **kwargs, 
):
    if headers := request_kwargs.get("headers"):
        headers = dict(headers)
    else:
        headers = {}
    headers.update(*args, **kwargs)
    request_kwargs["headers"] = headers


def escape_filename(
    s: str, 
    /, 
    table: dict[int, int | str] = {c: chr(c+65248) for c in b'"\\/:*?|><'}, # type: ignore
) -> str:
    return s.translate(table)


def items[K, V](
    m: Mapping[K, V] | Iterable[tuple[K, V]], 
    /, 
) -> Iterable[tuple[K, V]]:
    if isinstance(m, Mapping):
        try:
            get_items = getattr(m, "items")
            if isinstance((items := get_items()), ItemsView):
                return items
        except Exception:
            pass
        return ItemsView(m)
    return m


@overload
def check_response(resp: dict, /) -> dict:
    ...
@overload
def check_response(resp: Awaitable[dict], /) -> Coroutine[Any, Any, dict]:
    ...
def check_response(resp: dict | Awaitable[dict], /) -> dict | Coroutine[Any, Any, dict]:
    """检测 123 的某个接口的响应，如果成功则直接返回，否则根据具体情况抛出一个异常，基本上是 OSError 的实例
    """
    def check(resp, /) -> dict:
        if not isinstance(resp, dict) or resp.get("code", 0) not in (0, 200):
            raise P123OSError(EIO, resp)
        return resp
    if isawaitable(resp):
        async def check_await() -> dict:
            return check(await resp)
        return check_await()
    else:
        return check(resp)


class P123OpenClient:
    """123 网盘客户端

    .. admonition:: Reference

        https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced
    """

    def __init__(
        self, /, 
        client_id: str = "", 
        client_secret: str = "", 
        token: str = "", 
    ):
        self.client_id = client_id
        self.client_secret = client_secret
        self.token = token
        if client_id and client_secret:
            self.login_open()

    def __del__(self, /):
        self.close()

    @property
    def cookies(self, /):
        """请求所用的 Cookies 对象（同步和异步共用）
        """
        try:
            return self.__dict__["cookies"]
        except KeyError:
            from httpx import Cookies
            cookies = self.__dict__["cookies"] = Cookies()
            return cookies

    @property
    def cookiejar(self, /) -> CookieJar:
        """请求所用的 CookieJar 对象（同步和异步共用）
        """
        return self.cookies.jar

    @property
    def headers(self, /) -> MutableMapping:
        """请求头，无论同步还是异步请求都共用这个请求头
        """
        try:
            return self.__dict__["headers"]
        except KeyError:
            from multidict import CIMultiDict
            headers = self.__dict__["headers"] = CIMultiDict({
                "accept": "*/*", 
                "accept-encoding": "gzip, deflate", 
                "app-version": "3", 
                "connection": "keep-alive", 
                "platform": "open_platform", 
                "user-agent": "Mozilla/5.0 AppleWebKit/600 Safari/600 Chrome/124.0.0.0 Edg/124.0.0.0", 
            })
            return headers

    @locked_cacheproperty
    def session(self, /):
        """同步请求的 session 对象
        """
        import httpx_request
        from httpx import Client, HTTPTransport, Limits
        session = Client(
            limits=Limits(max_connections=256, max_keepalive_connections=64, keepalive_expiry=10), 
            transport=HTTPTransport(retries=5), 
            verify=False, 
        )
        setattr(session, "_headers", self.headers)
        setattr(session, "_cookies", self.cookies)
        return session

    @locked_cacheproperty
    def async_session(self, /):
        """异步请求的 session 对象
        """
        import httpx_request
        from httpx import AsyncClient, AsyncHTTPTransport, Limits
        session = AsyncClient(
            limits=Limits(max_connections=256, max_keepalive_connections=64, keepalive_expiry=10), 
            transport=AsyncHTTPTransport(retries=5), 
            verify=False, 
        )
        setattr(session, "_headers", self.headers)
        setattr(session, "_cookies", self.cookies)
        return session

    @property
    def token(self, /) -> str:
        return self._token

    @token.setter
    def token(self, value: str, /):
        self._token = value
        if value:
            self.headers["authorization"] = f"Bearer {self._token}"
        else:
            self.headers.pop("authorization", None)

    @token.deleter
    def token(self, /):
        self.token = ""

    def close(self, /) -> None:
        """删除 session 和 async_session 属性，如果它们未被引用，则应该会被自动清理
        """
        self.__dict__.pop("session", None)
        self.__dict__.pop("async_session", None)

    def request(
        self, 
        /, 
        url: str, 
        method: str = "GET", 
        request: None | Callable = None, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ):
        """执行 HTTP 请求，默认为 GET 方法
        """
        if not url.startswith(("http://", "https://")):
            url = complete_url(url, base_url)
        request_kwargs.setdefault("parse", default_parse)
        if request is None:
            request_kwargs["session"] = self.async_session if async_ else self.session
            return get_default_request()(
                url=url, 
                method=method, 
                async_=async_, 
                **request_kwargs, 
            )
        else:
            if headers := request_kwargs.get("headers"):
                request_kwargs["headers"] = {**self.headers, **headers}
            else:
                request_kwargs["headers"] = self.headers
            return request(
                url=url, 
                method=method, 
                **request_kwargs, 
            )

    @overload
    def login(
        self, 
        /, 
        client_id: str = "", 
        client_secret: str = "", 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def login(
        self, 
        /, 
        client_id: str = "", 
        client_secret: str = "", 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def login(
        self, 
        /, 
        client_id: str = "", 
        client_secret: str = "", 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """登录以获取 access_token

        :param client_id: 应用标识，创建应用时分配的 appId
        :param client_secret: 应用密钥，创建应用时分配的 secretId
        :param base_url: 接口的基地址
        :param async_: 是否异步
        :param request_kwargs: 其它请求参数

        :return: 接口的响应信息
        """
        if client_id:
            self.client_id = client_id
        else:
            client_id = self.client_id
        if client_id:
            self.client_secret = client_secret
        else:
            client_secret = self.client_secret
        def gen_step():
            resp = yield self.login_access_token_open( # type: ignore
                {"clientID": client_id, "clientSecret": client_secret}, 
                base_url=base_url, 
                async_=async_, 
                **request_kwargs, 
            )
            check_response(resp)
            self.token = resp["data"]["accessToken"]
            return resp
        return run_gen_step(gen_step, async_=async_)

    @overload
    @staticmethod
    def login_access_token(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    @staticmethod
    def login_access_token(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    @staticmethod
    def login_access_token(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取access_token

        POST https://open-api.123pan.com/api/v1/access_token

        .. attention::
            此接口有访问频率限制。请获取到 `access_token` 后本地保存使用，并在 `access_token `过期前及时重新获取。`access_token` 有效期根据返回的 "expiredAt" 字段判断。

        .. note::
            通过这种方式授权得到的 `access_token`，各个接口分别允许一个较低的 QPS

            /接入指南/开发者接入/开发须知

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/txgcvbfgh0gtuad5

        .. admonition:: Reference
            /接入指南/开发者接入/获取access_token

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/gn1nai4x0v0ry9ki

        :payload:
            - clientID: str     💡 应用标识，创建应用时分配的 appId
            - clientSecret: str 💡 应用密钥，创建应用时分配的 secretId
        """
        request_kwargs["url"] = complete_url("/api/v1/access_token", base_url)
        request_kwargs.setdefault("method", "POST")
        request_kwargs.setdefault("parse", default_parse)
        if request is None:
            request = get_default_request()
            request_kwargs["async_"] = async_
        return request(json=payload, **request_kwargs)

    @overload
    @staticmethod
    def login_auth(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    @staticmethod
    def login_auth(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    @staticmethod
    def login_auth(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """授权地址

        GET https://www.123pan.com/auth

        .. admonition:: Reference
            /接入指南/第三方挂载应用接入/授权地址

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/gr7ggimkcysm18ap

        :payload:
            - client_id: str    💡 应用标识，创建应用时分配的 appId
            - redirect_uri: str 💡 应用注册的回调地址
            - scope: str = "user:base,file:all:read,file:all:write" 💡 权限
            - state: str = ""   💡 自定义参数，任意取值
        """
        request_kwargs["url"] = complete_url("/auth", base_url)
        request_kwargs.setdefault("parse", default_parse)
        payload = dict_to_lower_merge(payload, scope="user:base,file:all:read,file:all:write")
        if request is None:
            request = get_default_request()
            request_kwargs["async_"] = async_
        return request(params=payload, **request_kwargs)

    @overload
    @staticmethod
    def login_refresh_token(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    @staticmethod
    def login_refresh_token(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    @staticmethod
    def login_refresh_token(
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = "https://www.123pan.com", 
        request: None | Callable = None, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """授权code获取access_token

        POST https://open-api.123pan.com/api/v1/oauth2/access_token

        .. note::
            通过这种方式授权得到的 `access_token`，各个接口分别允许更高的 QPS

            /接入指南/第三方挂载应用接入/授权须知

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/kf05anzt1r0qnudd

        .. admonition:: Reference
            /接入指南/第三方挂载应用接入/授权code获取access_token

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/gammzlhe6k4qtwd9

        :payload:
            - client_id: str        💡 应用标识，创建应用时分配的 appId
            - client_secret: str    💡 应用密钥，创建应用时分配的 secretId
            - code: str = <default> 💡 授权码
            - grant_type: "authorization_code" | "refresh_token" = <default> 💡 身份类型
            - redirect_uri: str = <default>  💡 应用注册的回调地址，`grant_type` 为 "authorization_code" 时必携带
            - refresh_token: str = <default> 💡 刷新 token，单次请求有效
        """
        request_kwargs["url"] = complete_url("/api/v1/oauth2/access_token", base_url)
        request_kwargs.setdefault("method", "POST")
        request_kwargs.setdefault("parse", default_parse)
        payload = dict_to_lower(payload)
        if not payload.get("grant_type"):
            if payload.get("refresh_token"):
                payload["grant_type"] = "refresh_token"
            else:
                payload["grant_type"] = "authorization_code"
        if request is None:
            request = get_default_request()
            request_kwargs["async_"] = async_
        return request(params=payload, **request_kwargs)

    @overload
    def dlink_disable(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def dlink_disable(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def dlink_disable(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """禁用直链空间

        POST https://open-api.123pan.com/api/v1/direct-link/disable

        .. admonition:: Reference
            /API列表/直链/禁用直链空间

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ccgz6fwf25nd9psl

        :payload:
            - fileID: int 💡 目录 id
        """
        api = complete_url("/api/v1/direct-link/disable", base_url)
        if not isinstance(payload, dict):
            payload = {"fileID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def dlink_enable(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def dlink_enable(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def dlink_enable(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """启用直链空间

        POST https://open-api.123pan.com/api/v1/direct-link/enable

        .. admonition:: Reference
            /API列表/直链/启用直链空间

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/cl3gvdmho288d376

        :payload:
            - fileID: int 💡 目录 id
        """
        api = complete_url("/api/v1/direct-link/enable", base_url)
        if not isinstance(payload, dict):
            payload = {"fileID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def dlink_log(
        self, 
        payload: dict | int = 1, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def dlink_log(
        self, 
        payload: dict | int = 1, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def dlink_log(
        self, 
        payload: dict | int = 1, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取直链日志

        GET https://open-api.123pan.com/api/v1/direct-link/log

        .. admonition:: Reference
            /API列表/直链/获取直链日志

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/agmqpmu0dm0iogc9

        :payload:
            - pageNum: int                           💡 第几页
            - pageSize: int  = 100                   💡 分页大小
            - startTime: str = "0001-01-01 00:00:00" 💡 开始时间，格式：YYYY-MM-DD hh:mm:ss
            - endTime: str.  = "9999-12-31 23:59:59" 💡 结束时间，格式：YYYY-MM-DD hh:mm:ss
        """
        api = complete_url("/api/v1/direct-link/log", base_url)
        if not isinstance(payload, dict):
            payload = {"pageNum": payload}
        payload = dict_to_lower_merge(payload, {
            "pageSize": 100, 
            "startTime": "0001-01-01 00:00:00", 
            "endTime": "9999-12-31 23:59:59", 
        })
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def dlink_m3u8(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def dlink_m3u8(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def dlink_m3u8(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取直链转码链接

        GET https://open-api.123pan.com/api/v1/direct-link/get/m3u8

        :payload:
            - fileID: int 💡 文件 id

        :return:
            响应数据的 data 字段是一个字典，键值如下：

            +---------------------+--------+----------+--------------------------------------------------------------+
            | 名称                | 类型   | 是否必填 | 说明                                                         |
            +=====================+========+==========+==============================================================+
            | list                | array  | 必填     | 响应列表                                                     |
            +---------------------+--------+----------+--------------------------------------------------------------|
            | list[*].resolutions | string | 必填     | 分辨率                                                       |
            +---------------------+--------+----------+--------------------------------------------------------------|
            | list[*].address     | string | 必填     | 播放地址。请将播放地址放入支持的 hls 协议的播放器中进行播放。|
            |                     |        |          | 示例在线播放地址: https://m3u8-player.com/                   |
            |                     |        |          | 请注意：转码链接播放过程中将会消耗您的直链流量。             |
            |                     |        |          | 如果您开启了直链鉴权,也需要将转码链接根据鉴权指引进行签名。  |
            +---------------------+--------+----------+--------------------------------------------------------------+
        """
        api = complete_url("/api/v1/direct-link/get/m3u8", base_url)
        if not isinstance(payload, dict):
            payload = {"fileID": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def dlink_transcode(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def dlink_transcode(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def dlink_transcode(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """发起直链转码

        POST https://open-api.123pan.com/api/v1/direct-link/doTranscode

        :payload:
            - ids: list[int] 💡 视频文件 id 列表
        """
        api = complete_url("/api/v1/direct-link/doTranscode", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"ids": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def dlink_transcode_query(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def dlink_transcode_query(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def dlink_transcode_query(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """查询直链转码进度

        POST https://open-api.123pan.com/api/v1/direct-link/queryTranscode

        :payload:
            - ids: str 💡 视频文件 id 列表

        :return:
            响应数据的 data 字段是一个字典，键值如下：

            +-----------+-------+----------+-------------------------------------------+
            | 名称      | 类型  | 是否必填 | 说明                                      |
            +===========+=======+==========+===========================================+
            | noneList  | array | 必填     | 未发起过转码的 ID                         |
            | errorList | array | 必填     | 错误文件ID列表,这些文件ID无法进行转码操作 |
            | success   | array | 必填     | 转码成功的文件ID列表                      |
            +-----------+-------+----------+-------------------------------------------+
        """
        api = complete_url("/api/v1/direct-link/queryTranscode", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"ids": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def dlink_url(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def dlink_url(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def dlink_url(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取直链链接

        GET https://open-api.123pan.com/api/v1/direct-link/url

        .. admonition:: Reference
            /API列表/直链/获取直链链接

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/tdxfsmtemp4gu4o2

        :payload:
            - fileID: int 💡 文件 id
        """
        api = complete_url("/api/v1/direct-link/url", base_url)
        if not isinstance(payload, dict):
            payload = {"fileID": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def download_info(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def download_info(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def download_info(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """下载

        GET https://open-api.123pan.com/api/v1/file/download_info

        .. admonition:: Reference
            /API列表/文件管理/下载

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/fnf60phsushn8ip2

        :payload:
            - fileId: int 💡 文件 id
        """
        api = complete_url("/api/v1/file/download_info", base_url)
        update_headers_in_kwargs(request_kwargs, platform="android")
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def fs_delete(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_delete(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_delete(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """彻底删除文件

        POST https://open-api.123pan.com/api/v1/file/delete

        .. attention::
            彻底删除文件前，文件必须要在回收站中，否则无法删除        

        .. admonition:: Reference
            /API列表/文件管理/删除/彻底删除文件

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/sg2gvfk5i3dwoxtg

        :payload:
            - fileIDs: list[int] 💡 文件 id 列表，最多 100 个
        """
        api = complete_url("/api/v1/file/delete", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIDs": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def fs_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取单个文件详情

        GET https://open-api.123pan.com/api/v1/file/detail

        .. admonition:: Reference
            /API列表/文件管理/文件详情/获取单个文件详情

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/owapsz373dzwiqbp

        :payload:
            - fileID: int 💡 文件 id
        """
        api = complete_url("/api/v1/file/detail", base_url)
        if not isinstance(payload, dict):
            payload = {"fileID": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def fs_info(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_info(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_info(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取多个文件详情

        POST https://open-api.123pan.com/api/v1/file/infos

        .. admonition:: Reference
            /API列表/文件管理/文件详情/获取多个文件详情

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/cqqayfuxybegrlru

        :payload:
            - fileIds: list[int] 💡 文件 id 列表
        """
        api = complete_url("/api/v1/file/infos", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIds": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def fs_list(
        self, 
        payload: dict | int | str = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_list(
        self, 
        payload: dict | int | str = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_list(
        self, 
        payload: dict | int | str = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取文件列表（推荐）

        GET https://open-api.123pan.com/api/v2/file/list

        .. admonition:: Reference
            /API列表/文件管理/文件列表/获取文件列表（推荐）

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/zrip9b0ye81zimv4

            /API列表/视频转码/上传视频/云盘上传/获取云盘视频文件

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/yqyi3rqrmrpvdf0d

            /API列表/视频转码/获取视频信息/获取转码空间文件列表

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ux9wct58lvllxm1n

        .. note::
            如果返回信息中，"lastFileId" 字段的值为 "-1"，代表最后一页（无需再翻页查询）。
            其它则代表下一页开始的文件 id，携带到请求参数中，可查询下一页

        :payload:
            - businessType: int = <default> 💡 业务类型：2:转码空间
            - category: int = <default>     💡 分类代码：0:未知 1:音频 2:视频 3:图片
            - lastFileId: int = <default>   💡 上一页的最后一条记录的 FileID，翻页查询时需要填写
            - limit: int = 100              💡 分页大小，最多 100
            - parentFileId: int | str = 0   💡 父目录 id，根目录是 0
            - searchData: str = <default>   💡 搜索关键字，将无视 `parentFileId`，而进行全局查找
            - searchMode: 0 | 1 = 0         💡 搜索模式

                - 0: 模糊搜索（将会根据搜索项分词，查找出相似的匹配项）
                - 1: 精准搜索（精准搜索需要提供完整的文件名）

            - trashed: "false" | "true" = "false" 💡 是否查看回收站的文件
        """
        api = complete_url("/api/v2/file/list", base_url)
        if isinstance(payload, (int, str)):
            payload = {"parentFileId": payload}
        payload = dict_to_lower_merge(payload, {
            "limit": 100, 
            "parentFileId": 0, 
            "searchMode": 0, 
            "trashed": "false", 
        })
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def fs_list_v1(
        self, 
        payload: dict | int | str = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_list_v1(
        self, 
        payload: dict | int | str = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_list_v1(
        self, 
        payload: dict | int | str = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取文件列表（旧）

        GET https://open-api.123pan.com/api/v1/file/list

        .. admonition:: Reference
            /API列表/文件管理/文件列表/获取文件列表（旧）

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/hosdqqax0knovnm2

        .. note::
            是否有下一页需要自行判断。如果返回的列表大小 < `limit`，或者根据返回值里的 "total"，如果 = `page * limit`，都说明没有下一页

        :payload:
            - limit: int = 100         💡 分页大小，最多 100
            - orderBy: str = "file_id" 💡 排序依据
            
                - "file_id": 文件 id
                - "file_name": 文件名
                - "size":  文件大小
                - "create_at": 创建时间
                - "update_at": 更新时间
                - "share_id": 分享 id
                - ...

            - orderDirection: "asc" | "desc" = "asc" 💡 排序顺序

                - "asc": 升序，从小到大
                - "desc": 降序，从大到小

            - page: int = 1               💡 第几页，从 1 开始（可传 0 或不传，视为 1）
            - parentFileId: int | str = 0 💡 父目录 id，根目录是 0
            - trashed: "false" | "true" = "false" 💡 是否查看回收站的文件
            - searchData: str = <default> 💡 搜索关键字（将无视 `parentFileId` 参数）
        """
        api = complete_url("/api/v1/file/list", base_url)
        if isinstance(payload, (int, str)):
            payload = {"parentFileId": payload}
        payload = dict_to_lower_merge(payload, {
            "limit": 100, 
            "orderBy": "file_id", 
            "orderDirection": "asc", 
            "page": 1, 
            "parentFileId": 0, 
            "trashed": "false", 
        })
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def fs_mkdir(
        self, 
        payload: dict | str, 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_mkdir(
        self, 
        payload: dict | str, 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_mkdir(
        self, 
        payload: dict | str, 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建目录

        POST https://open-api.123pan.com/upload/v1/file/mkdir

        .. admonition:: Reference
            /API列表/文件管理/上传/创建目录

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ouyvcxqg3185zzk4

        :payload:
            - name: str 💡 文件名，不能重名
            - parentID: int = 0 💡 父目录 id，根目录是 0
        """
        api = complete_url("/upload/v1/file/mkdir", base_url)
        if not isinstance(payload, dict):
            payload = {"name": payload}
        payload = dict_to_lower_merge(payload, parentID=parent_id)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def fs_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """移动

        POST https://open-api.123pan.com/api/v1/file/move

        .. admonition:: Reference
            /API列表/文件管理/移动

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/rsyfsn1gnpgo4m4f

        :payload:
            - fileIDs: list[int] 💡 文件 id 列表，最多 100 个
            - toParentFileID: int = 0 💡 要移动到的目标目录 id，根目录是 0
        """
        api = complete_url("/api/v1/file/move", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIDs": payload}
        payload = dict_to_lower_merge(payload, toParentFileID=parent_id)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def fs_recover(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_recover(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_recover(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """从回收站恢复文件

        POST https://open-api.123pan.com/api/v1/file/recover

        .. admonition:: Reference
            /API列表/文件管理/删除/从回收站恢复文件

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/kx9f8b6wk6g55uwy

        :payload:
            - fileIDs: list[int] 💡 文件 id 列表，最多 100 个
        """
        api = complete_url("/api/v1/file/recover", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIDs": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def fs_rename(
        self, 
        payload: dict | str | tuple[int | str, str] | Iterable[str | tuple[int | str, str]], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_rename(
        self, 
        payload: dict | str | tuple[int | str, str] | Iterable[str | tuple[int | str, str]], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_rename(
        self, 
        payload: dict | str | tuple[int | str, str] | Iterable[str | tuple[int | str, str]], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """批量文件重命名

        POST https://open-api.123pan.com/api/v1/file/rename

        .. admonition:: Reference
            /API列表/文件管理/重命名/批量文件重命名

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/imhguepnr727aquk

        :payload:
            - renameList: list[str] 💡 列表，每个成员的格式为 f"{fileId}|{fileName}"，最多 30 个
        """
        api = complete_url("/api/v1/file/rename", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, str):
                payload = [payload]
            elif isinstance(payload, tuple):
                payload = ["%s|%s" % payload]
            else:
                payload = [s if isinstance(s, str) else "%s|%s" % s for s in payload]
            payload = {"renameList": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def fs_rename_one(
        self, 
        payload: dict | str | tuple[int | str, str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_rename_one(
        self, 
        payload: dict | str | tuple[int | str, str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_rename_one(
        self, 
        payload: dict | str | tuple[int | str, str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """单个文件重命名

        PUT https://open-api.123pan.com/api/v1/file/name

        .. admonition:: Reference
            /API列表/文件管理/重命名/单个文件重命名

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ha6mfe9tteht5skc

        :payload:
            - fileId: int   💡 文件 id
            - fileName: str 💡 文件名
        """
        api = complete_url("/api/v1/file/name", base_url)
        if not isinstance(payload, dict):
            fid: int | str
            if isinstance(payload, str):
                fid, name = payload.split("|", 1)
            else:
                fid, name = payload
            payload = {"fileId": fid, "fileName": name}
        return self.request(api, "PUT", json=payload, async_=async_, **request_kwargs)

    @overload
    def fs_trash(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_trash(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_trash(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """删除文件至回收站

        POST https://open-api.123pan.com/api/v1/file/trash

        .. admonition:: Reference
            /API列表/文件管理/删除/删除文件至回收站

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/en07662k2kki4bo6

        :payload:
            - fileIDs: list[int] 💡 文件 id 列表，最多 100 个
        """
        api = complete_url("/api/v1/file/trash", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIDs": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def offline_download(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def offline_download(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def offline_download(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建离线下载任务

        POST https://open-api.123pan.com/api/v1/offline/download

        .. admonition:: Reference
            /API列表/离线下载/创建离线下载任务

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/he47hsq2o1xvgado

        :payload:
            - callBackUrl: str = <default> 💡 回调地址，任务结束时调用以推送通知，需要支持 POST 并接受 JSON 数据，格式为

                .. code:: js

                    {
                        url: string,     // 下载资源地址
                        status: 0 | 1,   // 是否失败
                        fileReason: str, // 失败原因
                        fileID: int,     // 成功后，该文件在云盘上的 id
                    }

            - dirID: int = <default> 💡 指定下载到的目录的 id。默认会下载到 "/来自:离线下载" 目录中
            - fileName: str = ""     💡 自定义文件名称
            - url: str               💡 下载链接，支持 http/https
        """
        api = complete_url("/api/v1/offline/download", base_url)
        if not isinstance(payload, dict):
            payload = {"url": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def offline_process(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def offline_process(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def offline_process(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取离线下载进度

        GET https://open-api.123pan.com/api/v1/offline/download/process

        .. admonition:: Reference
            /API列表/离线下载/获取离线下载进度

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/sclficr3t655pii5

        :payload:
            - taskID: int 💡 离线下载任务 id
        """
        api = complete_url("/api/v1/offline/download/process", base_url)
        if not isinstance(payload, dict):
            payload = {"taskID": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def oss_copy(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_copy(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_copy(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建复制任务

        POST https://open-api.123pan.com/api/v1/oss/source/copy

        .. note::
            说明：图床复制任务创建（可创建的任务数：3，fileIDs 长度限制：100，当前一个任务处理完后将会继续处理下个任务）
该接口将会复制云盘里的文件或目录对应的图片到对应图床目录，每次任务包含的图片总数限制 1000 张，图片格式：png, gif, jpeg, tiff, webp,jpg,tif,svg,bmp，图片大小限制：100M，文件夹层级限制：15层
如果图床目录下存在相同 etag、size 的图片将会视为同一张图片，将覆盖原图片

        .. admonition:: Reference
            /API列表/图床/复制云盘图片/创建复制任务

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/trahy3lmds4o0i3r

        :payload:
            - fileIDs: list[int]      💡 文件 id 列表
            - toParentFileID: int = 0 💡 要移动到的目标目录 id，默认为根目录
            - sourceType: int = 1     💡 复制来源：1:云盘
            - type: int = 1           💡 业务类型，固定为 1
        """
        api = complete_url("/api/v1/oss/source/copy", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIDs": payload}
        payload = dict_to_lower_merge(payload, {
            "toParentFileID": parent_id, 
            "sourceType": 1, 
            "type": 1, 
        })
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_copy_process(
        self, 
        payload: dict | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_copy_process(
        self, 
        payload: dict | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_copy_process(
        self, 
        payload: dict | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取复制任务详情

        GET https://open-api.123pan.com/api/v1/oss/source/copy/process

        .. admonition:: Reference
            /API列表/图床/复制云盘图片/获取复制任务详情

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/rissl4ewklaui4th

        :payload:
            - taskID: str 💡 复制任务 id
        """
        api = complete_url("/api/v1/oss/source/copy/process", base_url)
        if not isinstance(payload, dict):
            payload = {"taskID": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def oss_copy_fail(
        self, 
        payload: dict | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_copy_fail(
        self, 
        payload: dict | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_copy_fail(
        self, 
        payload: dict | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取复制失败文件列表

        GET https://open-api.123pan.com/api/v1/oss/source/copy/fail

        .. admonition:: Reference
            /API列表/图床/复制云盘图片/获取复制失败文件列表

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/tlug9od3xlw2w23v

        :payload:
            - taskID: str      💡 复制任务 id
            - limit: int = 100 💡 每页条数，最多 100 个
            - page: int = 1    💡 第几页
        """
        api = complete_url("/upload/v1/oss/file/mkdir", base_url)
        if not isinstance(payload, dict):
            payload = {"taskID": payload}
        payload = dict_to_lower_merge(payload, limit=100, page=1)
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def oss_delete(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_delete(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_delete(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """删除图片

        POST https://open-api.123pan.com/api/v1/oss/file/delete

        .. attention::
            彻底删除文件前，文件必须要在回收站中，否则无法删除        

        .. admonition:: Reference
            /API列表/图床/删除图片

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ef8yluqdzm2yttdn

        :payload:
            - fileIDs: list[int] 💡 文件 id 列表，最多 100 个
        """
        api = complete_url("/api/v1/oss/file/delete", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIDs": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取图片详情

        GET https://open-api.123pan.com/api/v1/oss/file/detail

        .. admonition:: Reference
            /API列表/图床/获取图片信息/获取图片详情

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/rgf2ndfaxc2gugp8

        :payload:
            - fileID: int 💡 文件 id
        """
        api = complete_url("/api/v1/oss/file/detail", base_url)
        if not isinstance(payload, dict):
            payload = {"fileID": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def oss_list(
        self, 
        payload: dict | int | str = "", 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_list(
        self, 
        payload: dict | int | str = "", 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_list(
        self, 
        payload: dict | int | str = "", 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取图片列表

        POST https://open-api.123pan.com/api/v1/oss/file/list

        .. note::
            如果返回信息中，"lastFileId" 字段的值为 "-1"，代表最后一页（无需再翻页查询）。
            其它则代表下一页开始的文件 id，携带到请求参数中，可查询下一页

        .. admonition:: Reference
            /API列表/图床/获取图片信息/获取图片列表

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/zayr72q8xd7gg4f8

        :payload:
            - endTime: int = <default>    💡 筛选结束时间，时间戳格式，单位：秒
            - lastFileId: int = <default> 💡 上一页的最后一条记录的 FileID，翻页查询时需要填写
            - limit: int = 100            💡 分页大小，最多 100
            - parentFileId: int | str = 0 💡 父目录 id，默认为根目录
            - startTime: int = <default>  💡 筛选开始时间，时间戳格式，单位：秒
            - type: int = 1               💡 业务类型，固定为 1
        """
        api = complete_url("/api/v1/oss/file/list", base_url)
        if isinstance(payload, (int, str)):
            payload = {"parentFileId": payload}
        payload = dict_to_lower_merge(payload, limit=100, type=1)
        return self.request(api, "POST", data=payload, async_=async_, **request_kwargs)

    @overload
    def oss_mkdir(
        self, 
        payload: dict | str, 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_mkdir(
        self, 
        payload: dict | str, 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_mkdir(
        self, 
        payload: dict | str, 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建目录

        POST https://open-api.123pan.com/upload/v1/oss/file/mkdir

        .. admonition:: Reference
            /API列表/图床/上传图片/创建目录

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/tpqqm04ocqwvonrk

        :payload:
            - name: str 💡 文件名，不能重名
            - parentID: int = 0 💡 父目录 id，默认为根目录
            - type: int = 1 💡 业务类型，固定为 1
        """
        api = complete_url("/upload/v1/oss/file/mkdir", base_url)
        if not isinstance(payload, dict):
            payload = {"name": payload}
        payload = dict_to_lower_merge(payload, parentID=parent_id, type=1)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = "", 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """移动图片

        POST https://open-api.123pan.com/api/v1/oss/file/move

        .. admonition:: Reference
            /API列表/图床/移动图片

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/eqeargimuvycddna

        :payload:
            - fileIDs: list[int] 💡 文件 id 列表，最多 100 个
            - toParentFileID: int = 0 💡 要移动到的目标目录 id，默认是根目录
        """
        api = complete_url("/api/v1/oss/file/move", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"fileIDs": payload}
        payload = dict_to_lower_merge(payload, toParentFileID=parent_id)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_offline_download(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_offline_download(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_offline_download(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建离线迁移任务

        POST https://open-api.123pan.com/api/v1/oss/offline/download

        .. admonition:: Reference
            /API列表/图床/图床离线迁移/创建离线迁移任务

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ctigc3a08lqzsfnq

        :payload:
            - businessDirID: int = <default> 💡 指定下载到的目录的 id。默认会下载到 "/来自:离线下载" 目录中
            - callBackUrl: str = <default> 💡 回调地址，任务结束时调用以推送通知，需要支持 POST 并接受 JSON 数据，格式为

                .. code:: js

                    {
                        url: string,     // 下载资源地址
                        status: 0 | 1,   // 是否失败
                        fileReason: str, // 失败原因
                        fileID: int,     // 成功后，该文件在云盘上的 id
                    }

            - fileName: str = "" 💡 自定义文件名称
            - type: int = 1 💡 业务类型，固定为 1
            - url: str 💡 下载链接，支持 http/https
        """
        api = complete_url("/api/v1/oss/offline/download", base_url)
        if not isinstance(payload, dict):
            payload = {"url": payload}
        payload = dict_to_lower_merge(payload, type=1)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_offline_process(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_offline_process(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_offline_process(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取离线迁移任务

        GET https://open-api.123pan.com/api/v1/oss/offline/download/process

        .. admonition:: Reference
            /API列表/图床/图床离线迁移/获取离线迁移任务

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/svo92desugbyhrgq

        :payload:
            - taskID: int 💡 离线下载任务 id
        """
        api = complete_url("/api/v1/oss/offline/download/process", base_url)
        if not isinstance(payload, dict):
            payload = {"taskID": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def oss_upload_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_upload_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_upload_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建文件

        POST https://open-api.123pan.com/upload/v1/oss/file/create

        .. note::
            - 文件名要小于 256 个字符且不能包含以下字符："\\/:*?|><
            - 文件名不能全部是空格
            - 不会重名

        .. admonition:: Reference
            /API列表/图床/上传图片/创建文件

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/xwfka5kt6vtmgs8r

        :payload:
            - filename: str 💡 文件名
            - duplicate: 0 | 1 | 2 = 0 💡 处理同名：0: 跳过/报错 1: 保留/后缀编号 2: 替换/覆盖
            - etag: str 💡 文件 md5
            - parentFileID: int = 0 💡 父目录 id，默认为根目录
            - size: int 💡 文件大小，单位：字节
            - type: int = 1 💡 业务类型，固定为 1

        :return:
            返回的数据说明如下：

            .. code:: python

                {
                    "fileID": str, # 上传后的文件 id。当已有相同 `size` 和 `etag` 的文件时，会发生秒传
                    "preuploadID": str, # 预上传 id。当 `reuse` 为 "true" 时，该字段不存在
                    "reuse": bool, # 是否秒传，返回 "true" 时表示文件已上传成功
                    "sliceSize": int, # 分片大小，必须按此大小生成文件分片再上传。当 `reuse` 为 "true" 时，该字段不存在
                }
        """
        api = complete_url("/upload/v1/oss/file/create", base_url)
        payload = dict_to_lower_merge(payload, type=1)
        if "duplicate" in payload and not payload["duplicate"]:
            del payload["duplicate"]
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_upload_url(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_upload_url(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_upload_url(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取上传地址&上传分片

        POST https://open-api.123pan.com/upload/v1/oss/file/get_upload_url

        .. note::
            有多个分片时，轮流分别根据序号获取下载链接，然后 PUT 方法上传分片。由于上传链接会过期，所以没必要提前获取一大批

        .. admonition:: Reference
            /API列表/图床/上传图片/获取上传地址&上传分片

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/pyfo3a39q6ac0ocd

        :payload:
            - preuploadID: str 💡 预上传 id
            - sliceNo: int     💡 分片序号，从 1 开始自增
        """
        api = complete_url("/upload/v1/oss/file/get_upload_url", base_url)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_upload_list(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_upload_list(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_upload_list(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """列举已上传分片

        POST https://open-api.123pan.com/upload/v1/oss/file/list_upload_parts

        .. note::
            此接口用于罗列已经上传的分片信息，以供比对

        :payload:
            - preuploadID: str 💡 预上传 id
        """
        api = complete_url("/upload/v1/oss/file/list_upload_parts", base_url)
        if not isinstance(payload, dict):
            payload = {"preuploadID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_upload_complete(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_upload_complete(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_upload_complete(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """上传完毕

        POST https://open-api.123pan.com/upload/v1/oss/file/upload_complete

        .. admonition:: Reference
            /API列表/图床/上传图片/上传完毕

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/yhgo0kt3nkngi8r2

        :payload:
            - preuploadID: str 💡 预上传 id

        :return:
            返回的数据说明如下：

            .. code:: python

                {
                    "async": bool, # 是否需要异步查询上传结果
                    "completed": bool, # 上传是否完成
                    "fileID": int, # 上传的文件 id
                }
        """
        api = complete_url("/upload/v1/oss/file/upload_complete", base_url)
        if not isinstance(payload, dict):
            payload = {"preuploadID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_upload_result(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_upload_result(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_upload_result(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """异步轮询获取上传结果

        POST https://open-api.123pan.com/upload/v1/oss/file/upload_async_result

        .. admonition:: Reference
            /API列表/图床/上传图片/异步轮询获取上传结果

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/lbdq2cbyzfzayipu

        :payload:
            - preuploadID: str 💡 预上传 id

        :return:
            返回的数据说明如下：

            .. code:: python

                {
                    "completed": bool, # 上传合并是否完成，如果为 False，请至少 1 秒后再发起轮询
                    "fileID": int, # 上传的文件 id
                }
        """
        api = complete_url("/upload/v1/oss/file/upload_async_result", base_url)
        if not isinstance(payload, dict):
            payload = {"preuploadID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def oss_upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = "", 
        duplicate: Literal[0, 1, 2] = 0, 
        preupload_id: None | str = None, 
        slice_size: int = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def oss_upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = "", 
        duplicate: Literal[0, 1, 2] = 0, 
        preupload_id: None | str = None, 
        slice_size: int = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def oss_upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = "", 
        duplicate: Literal[0, 1, 2] = 0, 
        preupload_id: None | str = None, 
        slice_size: int = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """上传文件

        .. note::
            如果文件名中包含字符 "\\/:*?|><，则转换为对应的全角字符

        .. admonition:: Reference
            /API列表/图床/上传图片/💡上传流程说明

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/di0url3qn13tk28t

        :param file: 待上传的文件

            - 如果为 `collections.abc.Buffer`，则作为二进制数据上传
            - 如果为 `filewrap.SupportsRead`，则作为可读的二进制文件上传
            - 如果为 `str` 或 `os.PathLike`，则视为路径，打开后作为文件上传
            - 如果为 `yarl.URL` 或 `http_request.SupportsGeturl` (`pip install python-http_request`)，则视为超链接，打开后作为文件上传
            - 如果为 `collections.abc.Iterable[collections.abc.Buffer]` 或 `collections.abc.AsyncIterable[collections.abc.Buffer]`，则迭代以获取二进制数据，逐步上传

        :param file_md5: 文件的 MD5 散列值
        :param file_name: 文件名
        :param file_size: 文件大小
        :param parent_id: 要上传的目标目录，默认为根目录
        :param duplicate: 处理同名：0: 提示/忽略 1: 保留两者 2: 替换
        :param preupload_id: 预上传 id，用于断点续传，提供此参数，则会忽略 `file_md5`、`file_name`、`file_size`、`parent_id` 和 `duplicate`
        :param slice_size: 分块大小，断点续传时，如果只上传过少于 2 个分块时，会被使用
        :param async_: 是否异步
        :param request_kwargs: 其它请求参数

        :return: 接口响应
        """
        def gen_step():
            nonlocal file, file_md5, file_name, file_size, preupload_id, slice_size
            def do_upload(file):
                return self.oss_upload_file_open(
                    file=file, 
                    file_md5=file_md5, 
                    file_name=file_name, 
                    file_size=file_size, 
                    parent_id=parent_id, 
                    duplicate=duplicate, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
            try:
                file = getattr(file, "getbuffer")()
            except (AttributeError, TypeError):
                pass
            if isinstance(file, Buffer):
                file_size = buffer_length(file)
                if not file_md5:
                    file_md5 = md5(file).hexdigest()
            elif isinstance(file, (str, PathLike)):
                path = fsdecode(file)
                if not file_name:
                    file_name = basename(path)
                if async_:
                    async def request():
                        async with async_open(path, "rb") as file:
                            setattr(file, "fileno", file.file.fileno)
                            setattr(file, "seekable", lambda: True)
                            return await do_upload(file)
                    return request
                else:
                    return do_upload(open(path, "rb"))
            elif isinstance(file, SupportsRead):
                seek = getattr(file, "seek", None)
                seekable = False
                curpos = 0
                if callable(seek):
                    if async_:
                        seek = ensure_async(seek, threaded=True)
                    try:
                        seekable = getattr(file, "seekable")()
                    except (AttributeError, TypeError):
                        try:
                            curpos = yield seek(0, 1)
                            seekable = True
                        except Exception:
                            seekable = False
                if not file_md5:
                    if not seekable:
                        fsrc = file
                        file = TemporaryFile()
                        if async_:
                            yield copyfileobj_async(fsrc, file)
                        else:
                            copyfileobj(fsrc, file)
                        file.seek(0)
                        return do_upload(file)
                    try:
                        if async_:
                            file_size, hashobj = yield file_digest_async(file)
                        else:
                            file_size, hashobj = file_digest(file)
                    finally:
                        yield cast(Callable, seek)(curpos)
                    file_md5 = hashobj.hexdigest()
                if file_size < 0:
                    try:
                        fileno = getattr(file, "fileno")()
                        file_size = fstat(fileno).st_size - curpos
                    except (AttributeError, TypeError, OSError):
                        try:
                            file_size = len(file) - curpos # type: ignore
                        except TypeError:
                            if seekable:
                                try:
                                    file_size = (yield cast(Callable, seek)(0, 2)) - curpos
                                finally:
                                    yield cast(Callable, seek)(curpos)
            elif isinstance(file, (URL, SupportsGeturl)):
                if isinstance(file, URL):
                    url = str(file)
                else:
                    url = file.geturl()
                if async_:
                    from httpfile import AsyncHttpxFileReader
                    async def request():
                        file = await AsyncHttpxFileReader.new(url)
                        async with file:
                            return await do_upload(file)
                    return request
                else:
                    from httpfile import HTTPFileReader
                    with HTTPFileReader(url) as file:
                        return do_upload(file)
            elif not file_md5 or file_size < 0:
                if async_:
                    file = bytes_iter_to_async_reader(file) # type: ignore
                else:
                    file = bytes_iter_to_reader(file) # type: ignore
                return do_upload(file)
            if not file_name:
                file_name = getattr(file, "name", "")
                file_name = basename(file_name)
            if file_name:
                file_name = escape_filename(file_name)
            else:
                file_name = str(uuid4())
            if file_size < 0:
                file_size = getattr(file, "length", 0)
            next_slice_no = 1
            if preupload_id:
                resp = yield self.oss_upload_list_open(
                    preupload_id, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
                check_response(resp)
                parts = resp["data"].get("parts")
                if not parts:
                    if not slice_size:
                        preupload_id = None
                elif len(parts) == 1:
                    if slice_size:
                        next_slice_no = slice_size == parts[0]["size"]
                    else:
                        warn("only one block was uploaded before, but it's not sure if it's complete", parts)
                        slice_size = parts[0]["size"]
                        next_slice_no = 2
                else:
                    slice_size = parts[0]["size"]
                    next_slice_no = len(parts) + (slice_size == parts[-1]["size"])
            if next_slice_no > 1:
                file_seek = getattr(file, "seek", None)
                if not callable(file_seek):
                    raise AttributeError(f"resume upload on an unseekable stream {file}")
                if async_:
                    file_seek = ensure_async(file_seek, threaded=True)
                yield file_seek(slice_size * (next_slice_no - 1))
            if not preupload_id:
                resp = yield self.oss_upload_create_open(
                    {
                        "etag": file_md5, 
                        "filename": file_name, 
                        "size": file_size, 
                        "parentFileID": parent_id, 
                        "duplicate": duplicate, 
                        "containDir": ("false", "true")[file_name.startswith("/")], 
                    }, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
                check_response(resp)
                upload_data = resp["data"]
                if upload_data["reuse"]:
                    return resp
                preupload_id = upload_data["preuploadID"]
                slice_size = int(upload_data["sliceSize"])
            upload_request_kwargs = {
                **request_kwargs, 
                "method": "PUT", 
                "headers": {"authorization": ""}, 
                "parse": ..., 
            }
            try:
                if async_:
                    async def request():
                        chunks = bio_chunk_async_iter(file, chunksize=slice_size) # type: ignore
                        slice_no = next_slice_no
                        async for chunk in chunks:
                            resp = await self.oss_upload_url_open(
                                {"preuploadID": preupload_id, "sliceNo": slice_no}, 
                                base_url=base_url, 
                                async_=True, 
                                **request_kwargs, 
                            )
                            check_response(resp)
                            upload_url = resp["data"]["presignedURL"]
                            await self.request(
                                upload_url, 
                                data=chunk, 
                                async_=True, 
                                **upload_request_kwargs, 
                            )
                            slice_no += 1
                    yield request()
                else:
                    chunks = bio_chunk_iter(file, chunksize=slice_size) # type: ignore
                    for slice_no, chunk in enumerate(chunks, next_slice_no):
                        resp = self.oss_upload_url_open(
                            {"preuploadID": preupload_id, "sliceNo": slice_no}, 
                            base_url=base_url, 
                            **request_kwargs, 
                        )
                        check_response(resp)
                        upload_url = resp["data"]["presignedURL"]
                        self.request(upload_url, data=chunk, **upload_request_kwargs)
                return (yield self.oss_upload_complete_open(
                    preupload_id, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                ))
            except BaseException as e:
                raise P123BrokenUpload({
                    "preupload_id": preupload_id, 
                    "file_md5": file_md5, 
                    "file_name": file_name, 
                    "file_size": file_size, 
                    "parent_id": parent_id, 
                    "duplicate": duplicate, 
                    "slice_size": slice_size, 
                }) from e
        return run_gen_step(gen_step, async_=async_)

    @overload
    def share_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建分享链接

        POST https://open-api.123pan.com/api/v1/share/create

        .. admonition:: Reference
            /API列表/分享管理/创建分享链接

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/dwd2ss0qnpab5i5s

        :payload:
            - fileIDList: str 💡 分享文件 id 列表，最多 100 个，用逗号,分隔连接
            - shareExpire: 0 | 1 | 7 | 30 = 0 💡 分享链接有效期天数，0 为永久
            - shareName: str 💡 分享链接名称，须小于 35 个字符且不能包含特殊字符 "\\/:*?|><
            - sharePwd: str = "" 💡 设置分享链接提取码
            - trafficLimit: int = <default> 💡 免登陆限制流量，单位：字节
            - trafficLimitSwitch: 1 | 2 = <default> 💡 免登录流量限制开关：1:关闭 2:打开
            - trafficSwitch: 1 | 2 = <default> 💡 免登录流量包开关：1:关闭 2:打开
        """
        api = complete_url("/api/v1/share/create", base_url)
        payload = dict_to_lower_merge(payload, {"shareExpire": 0, "sharePwd": ""})
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def share_create_paid(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_create_paid(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_create_paid(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建付费分享链接

        POST https://open-api.123pan.com/api/v1/share/content-payment/create

        .. admonition:: Reference
            /API列表/分享管理/创建付费分享链接

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/qz30c5k2npe8l98r

        :payload:
            - fileIDList: str        💡 分享文件 id 列表，最多 100 个，用逗号,分隔连接
            - isReward: 0 | 1 = 0    💡 是否开启打赏
            - payAmount: int = 1     💡 金额，从 1 到 99，单位：元
            - resourceDesc: str = "" 💡 资源描述
            - shareName: str         💡 分享链接名称，须小于 35 个字符且不能包含特殊字符 "\\/:*?|><
        """
        api = complete_url("/api/v1/share/content-payment/create", base_url)
        payload = dict_to_lower_merge(payload, {"payAmount": 1, "isReward": 0, "resourceDesc": ""})
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def share_edit(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_edit(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_edit(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """修改分享链接

        PUT https://open-api.123pan.com/api/v1/share/list/info

        .. admonition:: Reference
            /API列表/分享管理/修改分享链接

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ga6hhca1u8v9yqx0

        :payload:
            - shareIdList: list[int] 💡 分享链接 id 列表，最多 100 个
            - trafficLimit: int = <default> 💡 免登陆限制流量，单位：字节
            - trafficLimitSwitch: 1 | 2 = <default> 💡 免登录流量限制开关：1:关闭 2:打开
            - trafficSwitch: 1 | 2 = <default> 💡 免登录流量包开关：1:关闭 2:打开
        """
        api = complete_url("/api/v1/share/list/info", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                payload = [payload]
            elif not isinstance(payload, (tuple, list)):
                payload = list(payload)
            payload = {"shareIdList": payload}
        return self.request(api, "PUT", json=payload, async_=async_, **request_kwargs)

    @overload
    def share_list(
        self, 
        payload: dict | int = 100, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_list(
        self, 
        payload: dict | int = 100, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_list(
        self, 
        payload: dict | int = 100, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取分享链接列表

        GET https://open-api.123pan.com/api/v1/share/list

        .. admonition:: Reference
            /API列表/分享管理/获取分享链接列表

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ixg0arldi61fe7av

        :payload:
            - limit: int = 100     💡 每页文件数量，最多 100 个
            - lastShareId: int = 0 💡 从此分享 id 之后开始，默认为 0，即从头开始
        """
        api = complete_url("/api/v1/share/list", base_url)
        if not isinstance(payload, int):
            payload = {"limit": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_delete(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_delete(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_delete(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """删除转码视频

        POST https://open-api.123pan.com/api/v1/transcode/delete

        .. admonition:: Reference
            /API列表/视频转码/删除视频/删除转码视频

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/tg2xgotkgmgpulrp

        :payload:
            - fileId: int           💡 文件 id
            - businessType: int = 2 💡 业务类型：2:转码空间
            - trashed: int = 2      💡 删除范围：1:删除原文件 2:删除原文件+转码后的文件
        """
        api = complete_url("/api/v1/transcode/delete", base_url)
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        payload = dict_to_lower_merge(payload, businessType=2, trashed=2)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_download(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_download(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_download(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """原文件下载

        POST https://open-api.123pan.com/api/v1/transcode/file/download

        .. admonition:: Reference
            /API列表/视频转码/视频文件下载/原文件下载

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/mlltlx57sty6g9gf

        :payload:
            - fileId: int 💡 文件 id
        """
        api = complete_url("/api/v1/transcode/file/download", base_url)
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_download_all(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_download_all(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_download_all(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """某个视频全部转码文件下载

        POST https://open-api.123pan.com/api/v1/transcode/file/download/all

        .. attention::
            该接口需要轮询去查询结果，建议 10s 一次

        .. admonition:: Reference
            /API列表/视频转码/视频文件下载/某个视频全部转码文件下载

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/yb7hrb0x2gym7xic

        :payload:
            - fileId: int 💡 文件 id
            - zipName: str = f"转码{file_id}.zip" 💡 下载 zip 文件的名字
        """
        api = complete_url("/api/v1/transcode/file/download/all", base_url)
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        payload = dict_to_lower_merge(payload, zipName=f"转码{payload.get('fileid', '')}.zip")
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_m3u8_ts_download(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_m3u8_ts_download(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_m3u8_ts_download(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """单个转码文件下载（m3u8或ts）

        POST https://open-api.123pan.com/api/v1/transcode/m3u8_ts/download

        .. admonition:: Reference
            /API列表/视频转码/视频文件下载/单个转码文件下载（m3u8或ts）

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/yf97p60yyzb8mzbr

        :payload:
            - fileId: int     💡 文件 id
            - resolution: str 💡 分辨率
            - type: int       💡 文件类型：1:m3u8 2:ts
            - tsName: str     💡 下载 ts 文件时必须要指定名称，请参考查询某个视频的转码结果
        """
        api = complete_url("/api/v1/transcode/m3u8_ts/download", base_url)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取转码空间文件夹信息

        POST https://open-api.123pan.com/api/v1/transcode/folder/info

        .. admonition:: Reference
            /API列表/视频转码/获取视频信息/获取转码空间文件夹信息

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/kaalgke88r9y7nlt
        """
        api = complete_url("/api/v1/transcode/folder/info", base_url)
        return self.request(api, "POST", async_=async_, **request_kwargs)

    @overload
    def transcode_list(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_list(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_list(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """视频转码列表（三方挂载应用授权使用）

        GET https://open-api.123pan.com/api/v1/video/transcode/list

        .. attention::
            此接口仅限授权 `access_token` 调用

        .. admonition:: Reference
            /API列表/视频转码/获取视频信息/视频转码列表（三方挂载应用授权使用）

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/tgg6g84gdrmyess5

        :payload:
            - fileId: int 💡 文件 id
        """
        api = complete_url("/api/v1/video/transcode/list", base_url)
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        return self.request(api, params=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_record(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_record(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_record(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """查询某个视频的转码记录

        POST https://open-api.123pan.com/api/v1/transcode/video/record

        .. admonition:: Reference
            /API列表/视频转码/查询转码信息/查询某个视频的转码记录

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/ost1m82sa9chh0mc

        :payload:
            - fileId: int 💡 文件 id
        """
        api = complete_url("/api/v1/transcode/video/record", base_url)
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_resolutions(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_resolutions(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_resolutions(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取视频文件可转码的分辨率

        .. attention::
            该接口需要轮询去查询结果，建议 10s 一次

        POST https://open-api.123pan.com/api/v1/transcode/video/resolutions

        .. admonition:: Reference
            /API列表/视频转码/获取视频信息/获取视频文件可转码的分辨率

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/apzlsgyoggmqwl36

        :payload:
            - fileId: int 💡 文件 id
        """
        api = complete_url("/api/v1/transcode/video/resolutions", base_url)
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_result(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_result(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_result(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """查询某个视频的转码结果

        POST https://open-api.123pan.com/api/v1/transcode/video/result

        .. admonition:: Reference
            /API列表/视频转码/查询转码信息/查询某个视频的转码结果

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/iucbqgge0dgfc8sv

        :payload:
            - fileId: int 💡 文件 id
        """
        api = complete_url("/api/v1/transcode/video/result", base_url)
        if not isinstance(payload, dict):
            payload = {"fileId": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_upload(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_upload(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_upload(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """从云盘空间上传

        POST https://open-api.123pan.com/api/v1/transcode/upload/from_cloud_disk

        .. admonition:: Reference
            /API列表/视频转码/上传视频/云盘上传/从云盘空间上传

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/tqy2xatoo4qmdbz7

        :payload:
            - fileId: list[FileID] 💡 云盘空间文件 id，最多 100 个

                .. code:: python

                    FileID = {
                        "fileId": int # 文件 id
                    }
        """
        api = complete_url("/api/v1/transcode/upload/from_cloud_disk", base_url)
        if not isinstance(payload, dict):
            if isinstance(payload, (int, str)):
                fids = [{"fileId": payload}]
            else:
                fids = [{"fileId": fid} for fid in payload]
            payload = {"fileId": fids}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def transcode_video(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def transcode_video(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def transcode_video(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """视频转码操作

        POST https://open-api.123pan.com/api/v1/transcode/video

        .. admonition:: Reference
            /API列表/视频转码/视频转码/视频转码操作

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/xy42nv2x8wav9n5l

        :payload:
            - fileId: int      💡 文件 id
            - codecName: str   💡 编码方式
            - videoTime: int   💡 视频时长，单位：秒
            - resolutions: str 💡 要转码的分辨率（例如 1080P，P大写），多个用逗号,分隔连接，如："2160P,1080P,720P"
        """
        api = complete_url("/api/v1/transcode/video", base_url)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def upload_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_create(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建文件

        POST https://open-api.123pan.com/upload/v1/file/create

        .. note::
            - 文件名要小于 256 个字符且不能包含以下字符："\\/:*?|><
            - 文件名不能全部是空格
            - 开发者上传单文件大小限制 10 GB
            - 不会重名

        .. admonition:: Reference
            /API列表/文件管理/上传/V1/创建文件

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/lrfuu3qe7q1ul8ig

        :payload:
            - containDir: "false" | "true" = "false" 💡 上传文件是否包含路径
            - filename: str 💡 文件名，但 `containDir` 为 "true" 时，视为路径
            - duplicate: 0 | 1 | 2 = 0 💡 处理同名：0: 跳过/报错 1: 保留/后缀编号 2: 替换/覆盖
            - etag: str 💡 文件 md5
            - parentFileID: int = 0 💡 父目录 id，根目录是 0
            - size: int 💡 文件大小，单位：字节

        :return:
            返回的数据说明如下：

            .. code:: python

                {
                    "fileID": str, # 上传后的文件 id。当已有相同 `size` 和 `etag` 的文件时，会发生秒传
                    "preuploadID": str, # 预上传 id。当 `reuse` 为 "true" 时，该字段不存在
                    "reuse": bool, # 是否秒传，返回 "true" 时表示文件已上传成功
                    "sliceSize": int, # 分片大小，必须按此大小生成文件分片再上传。当 `reuse` 为 "true" 时，该字段不存在
                }
        """
        api = complete_url("/upload/v1/file/create", base_url)
        payload = dict_to_lower_merge(payload, {
            "parentFileId": 0, 
            "containDir": "false", 
        })
        if "duplicate" in payload and not payload["duplicate"]:
            del payload["duplicate"]
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def upload_url(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_url(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_url(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取上传地址&上传分片

        POST https://open-api.123pan.com/upload/v1/file/get_upload_url

        .. note::
            有多个分片时，轮流分别根据序号获取下载链接，然后 PUT 方法上传分片。由于上传链接会过期，所以没必要提前获取一大批

        .. admonition:: Reference
            /API列表/文件管理/上传/V1/获取上传地址&上传分片

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/sonz9n085gnz0n3m

        :payload:
            - preuploadID: str 💡 预上传 id
            - sliceNo: int     💡 分片序号，从 1 开始自增
        """
        api = complete_url("/upload/v1/file/get_upload_url", base_url)
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def upload_list(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_list(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_list(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """列举已上传分片

        POST https://open-api.123pan.com/upload/v1/file/list_upload_parts

        .. note::
            此接口用于罗列已经上传的分片信息，以供比对

        .. admonition:: Reference
            /API列表/文件管理/上传/V1/列举已上传分片（非必需）

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/dd28ws4bfn644cny

        :payload:
            - preuploadID: str 💡 预上传 id
        """
        api = complete_url("/upload/v1/file/list_upload_parts", base_url)
        if not isinstance(payload, dict):
            payload = {"preuploadID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def upload_complete(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_complete(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_complete(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """上传完毕

        POST https://open-api.123pan.com/upload/v1/file/upload_complete

        .. admonition:: Reference
            /API列表/文件管理/上传/V1/上传完毕

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/hkdmcmvg437rfu6x

        :payload:
            - preuploadID: str 💡 预上传 id

        :return:
            返回的数据说明如下：

            .. code:: python

                {
                    "async": bool, # 是否需要异步查询上传结果
                    "completed": bool, # 上传是否完成
                    "fileID": int, # 上传的文件 id
                }
        """
        api = complete_url("/upload/v1/file/upload_complete", base_url)
        if not isinstance(payload, dict):
            payload = {"preuploadID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    @overload
    def upload_result(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_result(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_result(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """异步轮询获取上传结果

        POST https://open-api.123pan.com/upload/v1/file/upload_async_result

        .. admonition:: Reference
            /API列表/文件管理/上传/V1/异步轮询获取上传结果

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/qgcosr6adkmm51h7

        :payload:
            - preuploadID: str 💡 预上传 id

        :return:
            返回的数据说明如下：

            .. code:: python

                {
                    "completed": bool, # 上传合并是否完成，如果为 False，请至少 1 秒后再发起轮询
                    "fileID": int, # 上传的文件 id
                }
        """
        api = complete_url("/upload/v1/file/upload_async_result", base_url)
        if not isinstance(payload, dict):
            payload = {"preuploadID": payload}
        return self.request(api, "POST", json=payload, async_=async_, **request_kwargs)

    # TODO: 如果已经有 md5 和 大小，则先尝试直接上传，而不是打开文件
    @overload
    def upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        preupload_id: None | str = None, 
        slice_size: int = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        preupload_id: None | str = None, 
        slice_size: int = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        preupload_id: None | str = None, 
        slice_size: int = 0, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """上传文件

        .. note::
            如果文件名中包含字符 "\\/:*?|><，则转换为对应的全角字符

        .. admonition:: Reference
            /API列表/文件管理/上传/v1/💡上传流程说明

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/il16qi0opiel4889

            /API列表/视频转码/上传视频/本地上传/上传流程

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/kh4ovskpumzn8r07

        :param file: 待上传的文件

            - 如果为 `collections.abc.Buffer`，则作为二进制数据上传
            - 如果为 `filewrap.SupportsRead`，则作为可读的二进制文件上传
            - 如果为 `str` 或 `os.PathLike`，则视为路径，打开后作为文件上传
            - 如果为 `yarl.URL` 或 `http_request.SupportsGeturl` (`pip install python-http_request`)，则视为超链接，打开后作为文件上传
            - 如果为 `collections.abc.Iterable[collections.abc.Buffer]` 或 `collections.abc.AsyncIterable[collections.abc.Buffer]`，则迭代以获取二进制数据，逐步上传

        :param file_md5: 文件的 MD5 散列值
        :param file_name: 文件名
        :param file_size: 文件大小
        :param parent_id: 要上传的目标目录
        :param duplicate: 处理同名：0: 提示/忽略 1: 保留两者 2: 替换
        :param preupload_id: 预上传 id，用于断点续传，提供此参数，则会忽略 `file_md5`、`file_name`、`file_size`、`parent_id` 和 `duplicate`
        :param slice_size: 分块大小，断点续传时，如果只上传过少于 2 个分块时，会被使用
        :param async_: 是否异步
        :param request_kwargs: 其它请求参数

        :return: 接口响应
        """
        def gen_step():
            nonlocal file, file_md5, file_name, file_size, preupload_id, slice_size
            def do_upload(file):
                return self.upload_file_open(
                    file=file, 
                    file_md5=file_md5, 
                    file_name=file_name, 
                    file_size=file_size, 
                    parent_id=parent_id, 
                    duplicate=duplicate, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
            try:
                file = getattr(file, "getbuffer")()
            except (AttributeError, TypeError):
                pass
            if isinstance(file, Buffer):
                file_size = buffer_length(file)
                if not file_md5:
                    file_md5 = md5(file).hexdigest()
            elif isinstance(file, (str, PathLike)):
                path = fsdecode(file)
                if not file_name:
                    file_name = basename(path)
                if async_:
                    async def request():
                        async with async_open(path, "rb") as file:
                            setattr(file, "fileno", file.file.fileno)
                            setattr(file, "seekable", lambda: True)
                            return await do_upload(file)
                    return request
                else:
                    return do_upload(open(path, "rb"))
            elif isinstance(file, SupportsRead):
                seek = getattr(file, "seek", None)
                seekable = False
                curpos = 0
                if callable(seek):
                    if async_:
                        seek = ensure_async(seek, threaded=True)
                    try:
                        seekable = getattr(file, "seekable")()
                    except (AttributeError, TypeError):
                        try:
                            curpos = yield seek(0, 1)
                            seekable = True
                        except Exception:
                            seekable = False
                if not file_md5:
                    if not seekable:
                        fsrc = file
                        file = TemporaryFile()
                        if async_:
                            yield copyfileobj_async(fsrc, file)
                        else:
                            copyfileobj(fsrc, file)
                        file.seek(0)
                        return do_upload(file)
                    try:
                        if async_:
                            file_size, hashobj = yield file_digest_async(file)
                        else:
                            file_size, hashobj = file_digest(file)
                    finally:
                        yield cast(Callable, seek)(curpos)
                    file_md5 = hashobj.hexdigest()
                if file_size < 0:
                    try:
                        fileno = getattr(file, "fileno")()
                        file_size = fstat(fileno).st_size - curpos
                    except (AttributeError, TypeError, OSError):
                        try:
                            file_size = len(file) - curpos # type: ignore
                        except TypeError:
                            if seekable:
                                try:
                                    file_size = (yield cast(Callable, seek)(0, 2)) - curpos
                                finally:
                                    yield cast(Callable, seek)(curpos)
            elif isinstance(file, (URL, SupportsGeturl)):
                if isinstance(file, URL):
                    url = str(file)
                else:
                    url = file.geturl()
                if async_:
                    from httpfile import AsyncHttpxFileReader
                    async def request():
                        file = await AsyncHttpxFileReader.new(url)
                        async with file:
                            return await do_upload(file)
                    return request
                else:
                    from httpfile import HTTPFileReader
                    with HTTPFileReader(url) as file:
                        return do_upload(file)
            elif not file_md5 or file_size < 0:
                if async_:
                    file = bytes_iter_to_async_reader(file) # type: ignore
                else:
                    file = bytes_iter_to_reader(file) # type: ignore
                return do_upload(file)
            if not file_name:
                file_name = getattr(file, "name", "")
                file_name = basename(file_name)
            if file_name:
                file_name = escape_filename(file_name)
            else:
                file_name = str(uuid4())
            if file_size < 0:
                file_size = getattr(file, "length", 0)
            next_slice_no = 1
            if preupload_id:
                resp = yield self.upload_list_open(
                    preupload_id, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
                check_response(resp)
                parts = resp["data"].get("parts")
                if not parts:
                    if not slice_size:
                        preupload_id = None
                elif len(parts) == 1:
                    if slice_size:
                        next_slice_no = slice_size == parts[0]["size"]
                    else:
                        warn("only one block was uploaded before, but it's not sure if it's complete", parts)
                        slice_size = parts[0]["size"]
                        next_slice_no = 2
                else:
                    slice_size = parts[0]["size"]
                    next_slice_no = len(parts) + (slice_size == parts[-1]["size"])
            if next_slice_no > 1:
                file_seek = getattr(file, "seek", None)
                if not callable(file_seek):
                    raise AttributeError(f"resume upload on an unseekable stream {file}")
                if async_:
                    file_seek = ensure_async(file_seek, threaded=True)
                yield file_seek(slice_size * (next_slice_no - 1))
            if not preupload_id:
                resp = yield self.upload_create_open(
                    {
                        "etag": file_md5, 
                        "filename": file_name, 
                        "size": file_size, 
                        "parentFileID": parent_id, 
                        "duplicate": duplicate, 
                        "containDir": ("false", "true")[file_name.startswith("/")], 
                    }, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
                check_response(resp)
                upload_data = resp["data"]
                if upload_data["reuse"]:
                    return resp
                preupload_id = upload_data["preuploadID"]
                slice_size = int(upload_data["sliceSize"])
            upload_request_kwargs = {
                **request_kwargs, 
                "method": "PUT", 
                "headers": {"authorization": ""}, 
                "parse": ..., 
            }
            try:
                if async_:
                    async def request():
                        chunks = bio_chunk_async_iter(file, chunksize=slice_size) # type: ignore
                        slice_no = next_slice_no
                        async for chunk in chunks:
                            resp = await self.upload_url_open(
                                {"preuploadID": preupload_id, "sliceNo": slice_no}, 
                                base_url=base_url, 
                                async_=True, 
                                **request_kwargs, 
                            )
                            check_response(resp)
                            upload_url = resp["data"]["presignedURL"]
                            await self.request(
                                upload_url, 
                                data=chunk, 
                                async_=True, 
                                **upload_request_kwargs, 
                            )
                            slice_no += 1
                    yield request()
                else:
                    chunks = bio_chunk_iter(file, chunksize=slice_size) # type: ignore
                    for slice_no, chunk in enumerate(chunks, next_slice_no):
                        resp = self.upload_url_open(
                            {"preuploadID": preupload_id, "sliceNo": slice_no}, 
                            base_url=base_url, 
                            **request_kwargs, 
                        )
                        check_response(resp)
                        upload_url = resp["data"]["presignedURL"]
                        self.request(upload_url, data=chunk, **upload_request_kwargs)
                return (yield self.upload_complete_open(
                    preupload_id, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                ))
            except BaseException as e:
                raise P123BrokenUpload({
                    "preupload_id": preupload_id, 
                    "file_md5": file_md5, 
                    "file_name": file_name, 
                    "file_size": file_size, 
                    "parent_id": parent_id, 
                    "duplicate": duplicate, 
                    "slice_size": slice_size, 
                }) from e
        return run_gen_step(gen_step, async_=async_)

    @overload
    def user_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def user_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def user_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_OPEN_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取用户信息

        GET https://open-api.123pan.com/api/v1/user/info

        .. admonition:: Reference
            /API列表/用户管理/获取用户信息

            https://123yunpan.yuque.com/org-wiki-123yunpan-muaork/cr6ced/fa2w0rosunui2v4m

        :payload:
            - preuploadID: str 💡 预上传 id

        :return:
            返回的数据说明如下：

            .. code:: python

                {
                    "async": bool, # 是否需要异步查询上传结果
                    "completed": bool, # 上传是否完成
                    "fileID": int, # 上传的文件 id
                }
        """
        api = complete_url("/api/v1/user/info", base_url)
        return self.request(api, async_=async_, **request_kwargs)

    login_open = login
    login_access_token_open = login_access_token
    login_auth_open = login_auth
    login_refresh_token_open = login_refresh_token
    dlink_disable_open = dlink_disable
    dlink_enable_open = dlink_enable
    dlink_log_open = dlink_log
    dlink_m3u8_open = dlink_m3u8
    dlink_transcode_open = dlink_transcode
    dlink_transcode_query_open = dlink_transcode_query
    dlink_url_open = dlink_url
    download_info_open = download_info
    fs_delete_open = fs_delete
    fs_detail_open = fs_detail
    fs_info_open = fs_info
    fs_list_open = fs_list
    fs_list_v1_open = fs_list_v1
    fs_mkdir_open = fs_mkdir
    fs_move_open = fs_move
    fs_recover_open = fs_recover
    fs_rename_open = fs_rename
    fs_rename_one_open = fs_rename_one
    fs_trash_open = fs_trash
    offline_download_open = offline_download
    offline_process_open = offline_process
    oss_copy_open = oss_copy
    oss_copy_fail_open = oss_copy_fail
    oss_copy_process_open = oss_copy_process
    oss_delete_open = oss_delete
    oss_detail_open = oss_detail
    oss_list_open = oss_list
    oss_mkdir_open = oss_mkdir
    oss_move_open = oss_move
    oss_offline_download_open = oss_offline_download
    oss_offline_process_open = oss_offline_process
    oss_upload_complete_open = oss_upload_complete
    oss_upload_create_open = oss_upload_create
    oss_upload_file_open = oss_upload_file
    oss_upload_list_open = oss_upload_list
    oss_upload_result_open = oss_upload_result
    oss_upload_url_open = oss_upload_url
    share_create_open = share_create
    share_create_paid_open = share_create_paid
    share_edit_open = share_edit
    share_list_open = share_list
    transcode_delete_open = transcode_delete
    transcode_download_open = transcode_download
    transcode_download_all_open = transcode_download_all
    transcode_m3u8_ts_download_open = transcode_m3u8_ts_download
    transcode_info_open = transcode_info
    transcode_list_open = transcode_list
    transcode_record_open = transcode_record
    transcode_resolutions_open = transcode_resolutions
    transcode_result_open = transcode_result
    transcode_upload_open = transcode_upload
    transcode_video_open = transcode_video
    upload_complete_open = upload_complete
    upload_create_open = upload_create
    upload_file_open = upload_file
    upload_list_open = upload_list
    upload_result_open = upload_result
    upload_url_open = upload_url
    user_info_open = user_info


class P123Client(P123OpenClient):

    def __init__(
        self, 
        /, 
        passport: int | str = "", 
        password: str = "", 
        token: str = "", 
    ):
        self.passport = passport
        self.password = password
        self.token = token
        if passport and password:
            self.login()

    @overload # type: ignore
    def login(
        self, 
        /, 
        passport: int | str = "", 
        password: str = "", 
        remember: bool = True, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def login(
        self, 
        /, 
        passport: int | str = "", 
        password: str = "", 
        remember: bool = True, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def login(
        self, 
        /, 
        passport: int | str = "", 
        password: str = "", 
        remember: bool = True, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """登录以获取 access_token

        :param passport: 账号
        :param password: 密码
        :param remember: 是否记住密码（不用管）
        :param base_url: 接口的基地址
        :param async_: 是否异步
        :param request_kwargs: 其它请求参数

        :return: 接口的响应信息
        """
        if passport:
            self.passport = passport
        else:
            passport = self.passport
        if password:
            self.password = password
        else:
            password = self.password
        def gen_step():
            resp = yield self.user_login(
                {"passport": passport, "password": password, "remember": remember}, 
                base_url=base_url, 
                async_=async_, 
                **request_kwargs, 
            )
            check_response(resp)
            self.token = resp["data"]["token"]
            return resp
        return run_gen_step(gen_step, async_=async_)

    @overload
    @staticmethod
    def app_dydomain(
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        request: None | Callable = None, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    @staticmethod
    def app_dydomain(
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        request: None | Callable = None, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    @staticmethod
    def app_dydomain(
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        request: None | Callable = None, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取 123 网盘的各种域名

        GET https://www.123pan.com/api/dydomain
        """
        request_kwargs["url"] = complete_url("/api/dydomain", base_url)
        request_kwargs.setdefault("parse", default_parse)
        if request is None:
            request = get_default_request()
            request_kwargs["async_"] = async_
        return request(**request_kwargs)

    @overload
    def download_info(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def download_info(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def download_info(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取下载信息

        POST https://www.123pan.com/api/file/download_info

        .. hint::
            即使文件已经被删除，只要还有 S3KeyFlag 和 Etag （即 MD5） 就依然可以下载

            你完全可以构造这样的查询参数

            .. code:: python

                payload = {
                    "Etag": "...", # 必填，文件的 MD5
                    "FileID": 0, # 可以随便填
                    "FileName": "a", # 随便填一个名字
                    "S3KeyFlag": str # 必填，格式为 f"{UID}-0"，UID 就是上传此文件的用户的 UID，如果此文件是由你上传的，则可从 `P123Client.user_info` 的响应中获取
                    "Size": 0, # 可以随便填，填了可能搜索更准确
                }

        .. note::
            获取的直链有效期是 24 小时

        :payload:
            - Etag: str 💡 文件的 MD5 散列值
            - S3KeyFlag: str
            - FileName: str = <default> 💡 默认用 Etag（即 MD5）作为文件名
            - FileID: int | str = 0
            - Size: int = <default>
            - Type: int = 0
            - driveId: int | str = 0
            - ...
        """
        def gen_step():
            nonlocal payload
            update_headers_in_kwargs(request_kwargs, platform="android")
            if not isinstance(payload, dict):
                resp = yield self.fs_info(
                    payload, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
                resp["payload"] = payload
                check_response(resp)
                info_list = resp["data"]["infoList"]
                if not info_list:
                    raise FileNotFoundError(ENOENT, resp)
                payload = cast(dict, info_list[0])
                if payload["Type"]:
                    raise IsADirectoryError(EISDIR, resp)
            payload = dict_to_lower_merge(
                payload, {"driveId": 0, "Type": 0, "FileID": 0})
            if "filename" not in payload:
                payload["filename"] = payload["etag"]
            return self.request(
                "file/download_info", 
                "POST", 
                json=payload, 
                base_url=base_url, 
                async_=async_, 
                **request_kwargs, 
            )
        return run_gen_step(gen_step, async_=async_)

    @overload
    def download_info_batch(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def download_info_batch(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def download_info_batch(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取批量下载信息

        POST https://www.123pan.com/api/file/batch_download_info

        .. warning::
            会把一些文件或目录以 zip 包的形式下载，但非会员有流量限制，所以还是推荐用 `P123Client.download_info` 逐个获取下载链接并下载

        :payload:
            - fileIdList: list[FileID]

                .. code:: python

                    FileID = {
                        "FileId": int | str
                    }
        """
        if isinstance(payload, (int, str)):
            payload = {"fileIdList": [{"FileId": payload}]}
        elif not isinstance(payload, dict):
            payload = {"fileIdList": [{"FileId": fid} for fid in payload]}
        return self.request(
            "file/batch_download_info", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def download_url(
        self, 
        payload: dict | int | str, 
        /, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> str:
        ...
    @overload
    def download_url(
        self, 
        payload: dict | int | str, 
        /, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, str]:
        ...
    def download_url(
        self, 
        payload: dict | int | str, 
        /, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> str | Coroutine[Any, Any, str]:
        """获取下载链接

        :params payload: 文件 id 或者文件信息，文件信息必须包含的信息如下：

            - S3KeyFlag: str            💡 存储桶名
            - Etag: str                 💡 文件的 MD5 散列值
            - Size: int                 💡 文件大小
            - FileName: str = <default> 💡 默认用 Etag（即 MD5）作为文件名

        :params async_: 是否异步
        :params request_kwargs: 其它请求参数

        :return: 下载链接
        """
        def gen_step():
            if isinstance(payload, dict):
                resp = yield self.download_info(
                    payload, 
                    async_=async_, 
                    **request_kwargs, 
                )
                check_response(resp)
                return resp["data"]["DownloadUrl"]
            else:
                resp = yield self.download_info_open(
                    payload, 
                    async_=async_, 
                    **request_kwargs, 
                )
                check_response(resp)
                return resp["data"]["downloadUrl"]
        return run_gen_step(gen_step, async_=async_)

    @overload
    def fs_copy(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_copy(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_copy(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """复制

        POST https://www.123pan.com/api/restful/goapi/v1/file/copy/async

        :payload:
            - fileList: list[File] 💡 信息可以取自 `P123Client.fs_info` 接口

                .. code:: python

                    File = { 
                        "FileId": int | str, 
                        ...
                    }

            - targetFileId: int | str = 0
        """
        def gen_step():
            nonlocal payload
            if not isinstance(payload, dict):
                resp = yield self.fs_info(
                    payload, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
                resp["payload"] = payload
                check_response(resp)
                info_list = resp["data"]["infoList"]
                if not info_list:
                    raise FileNotFoundError(ENOENT, resp)
                payload = {"fileList": info_list}
            payload = dict_to_lower_merge(payload, targetFileId=parent_id)
            return self.request(
                "restful/goapi/v1/file/copy/async", 
                "POST", 
                json=payload, 
                base_url=base_url, 
                async_=async_, 
                **request_kwargs, 
            )
        return run_gen_step(gen_step, async_=async_)

    @overload
    def fs_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_detail(
        self, 
        payload: dict | int | str, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取文件或目录详情（文件数、目录数、总大小）

        GET https://www.123pan.com/api/file/detail

        :payload:
            - fileID: int | str
        """
        if isinstance(payload, (int, str)):
            payload = {"fileID": payload}
        return self.request(
            "file/detail", 
            params=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def fs_delete(
        self, 
        payload: dict | int | str | Iterable[int | str] = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_delete(
        self, 
        payload: dict | int | str | Iterable[int | str] = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_delete(
        self, 
        payload: dict | int | str | Iterable[int | str] = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """彻底删除

        POST https://www.123pan.com/api/file/delete

        .. hint::
            彻底删除文件前,文件必须要在回收站中,否则无法删除

        :payload:
            - fileIdList: list[FileID]

                .. code:: python

                    FileID = {
                        "FileId": int | str
                    }

            - event: str = "recycleDelete"
        """
        if isinstance(payload, (int, str)):
            payload = {"fileIdList": [{"FileId": payload}]}
        elif not isinstance(payload, dict):
            payload = {"fileIdList": [{"FileId": fid} for fid in payload]}
        payload = cast(dict, payload)
        payload.setdefault("event", "recycleDelete")
        return self.request(
            "file/delete", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def fs_get_path(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_get_path(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_get_path(
        self, 
        payload: dict | int, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取某个 id 对应的祖先节点列表

        POST https://www.123pan.com/api/file/get_path

        :payload:
            - fileId: int 💡 文件 id
        """
        if isinstance(payload, int):
            payload = {"fileId": payload}
        return self.request(
            "file/get_path", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def fs_info(
        self, 
        payload: dict | int | str | Iterable[int | str] = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_info(
        self, 
        payload: dict | int | str | Iterable[int | str] = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_info(
        self, 
        payload: dict | int | str | Iterable[int | str] = 0, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取文件信息

        POST https://www.123pan.com/api/file/info

        :payload:
            - fileIdList: list[FileID]

                .. code:: python

                    FileID = {
                        "FileId": int | str
                    }
        """
        if isinstance(payload, (int, str)):
            payload = {"fileIdList": [{"FileId": payload}]}
        elif not isinstance(payload, dict):
            payload = {"fileIdList": [{"FileId": fid} for fid in payload]}
        return self.request(
            "file/info", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload # type: ignore
    def fs_list(
        self, 
        payload: dict | int | str = 0, 
        /, 
        event: str = "homeListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_list(
        self, 
        payload: dict | int | str = 0, 
        /, 
        event: str = "homeListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_list(
        self, 
        payload: dict | int | str = 0, 
        /, 
        event: str = "homeListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取文件列表（可搜索）

        GET https://www.123pan.com/api/file/list

        .. note::
            如果返回信息中，"Next" 字段的值为 "-1"，代表最后一页（无需再翻页查询）

        :payload:
            - driveId: int | str = 0
            - limit: int = 100 💡 分页大小，最多 100 个
            - next: int = 0    💡 下一批拉取开始的 id
            - orderBy: str = "file_id" 💡 排序依据

                - "file_id": 文件 id
                - "file_name": 文件名
                - "size":  文件大小
                - "create_at": 创建时间
                - "update_at": 更新时间
                - "share_id": 分享 id
                - ...

            - orderDirection: "asc" | "desc" = "asc" 💡 排序顺序
            - Page: int = <default> 💡 第几页，从 1 开始，可以是 0
            - parentFileId: int | str = 0 💡 父目录 id
            - trashed: "false" | "true" = <default> 💡 是否查看回收站的文件
            - inDirectSpace: "false" | "true" = "false"
            - event: str = "homeListFile" 💡 事件名称

                - "homeListFile": 全部文件
                - "recycleListFile": 回收站
                - "syncFileList": 同步空间

            - operateType: int | str = <default> 💡 操作类型，如果在同步空间，则需要指定为 "SyncSpacePage"
            - SearchData: str = <default> 💡 搜索关键字（将无视 `parentFileId` 参数）
            - OnlyLookAbnormalFile: int = <default>
        """
        if isinstance(payload, (int, str)):
            payload = {"parentFileId": payload}
        payload = dict_to_lower_merge(payload, {
            "driveId": 0, 
            "limit": 100, 
            "next": 0, 
            "orderBy": "file_id", 
            "orderDirection": "asc", 
            "parentFileId": 0, 
            "inDirectSpace": "false", 
            "event": event, 
        })
        if not payload.get("trashed"):
            match payload["event"]:
                case "recycleListFile":
                    payload["trashed"] = "true"
                case _:
                    payload["trashed"] = "false"
        return self.request(
            "file/list", 
            params=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def fs_list_new(
        self, 
        payload: dict | int | str = 0, 
        /, 
        event: str = "homeListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_list_new(
        self, 
        payload: dict | int | str = 0, 
        /, 
        event: str = "homeListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_list_new(
        self, 
        payload: dict | int | str = 0, 
        /, 
        event: str = "homeListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取文件列表（可搜索）

        GET https://www.123pan.com/api/file/list/new

        .. note::
            如果返回信息中，"Next" 字段的值为 "-1"，代表最后一页（无需再翻页查询）

        :payload:
            - driveId: int | str = 0
            - limit: int = 100 💡 分页大小，最多 100 个
            - next: int = 0    💡 下一批拉取开始的 id
            - orderBy: str = "file_id" 💡 排序依据

                - "file_id": 文件 id
                - "file_name": 文件名
                - "size":  文件大小
                - "create_at": 创建时间
                - "update_at": 更新时间
                - "share_id": 分享 id
                - ...

            - orderDirection: "asc" | "desc" = "asc" 💡 排序顺序
            - Page: int = <default> 💡 第几页，从 1 开始，可以是 0
            - parentFileId: int | str = 0 💡 父目录 id
            - trashed: "false" | "true" = <default> 💡 是否查看回收站的文件
            - inDirectSpace: "false" | "true" = "false"
            - event: str = "homeListFile" 💡 事件名称

                - "homeListFile": 全部文件
                - "recycleListFile": 回收站
                - "syncFileList": 同步空间

            - operateType: int | str = <default> 💡 操作类型，如果在同步空间，则需要指定为 "SyncSpacePage"
            - SearchData: str = <default> 💡 搜索关键字（将无视 `parentFileId` 参数）
            - OnlyLookAbnormalFile: int = <default>
        """
        if isinstance(payload, (int, str)):
            payload = {"parentFileId": payload}
        payload = dict_to_lower_merge(payload, {
            "driveId": 0, 
            "limit": 100, 
            "next": 0, 
            "orderBy": "file_id", 
            "orderDirection": "asc", 
            "parentFileId": 0, 
            "inDirectSpace": "false", 
            "event": event, 
        })
        if not payload.get("trashed"):
            match payload["event"]:
                case "recycleListFile":
                    payload["trashed"] = "true"
                case _:
                    payload["trashed"] = "false"
        return self.request(
            "file/list/new", 
            params=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload # type: ignore
    def fs_mkdir(
        self, 
        name: str, 
        /, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_mkdir(
        self, 
        name: str, 
        /, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_mkdir(
        self, 
        name: str, 
        /, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建目录

        :param name: 目录名
        :param parent_id: 父目录 id
        :param duplicate: 处理同名：0: 复用 1: 保留两者 2: 替换
        :param async_: 是否异步
        :param request_kwargs: 其它请求参数

        :return: 接口响应
        """
        payload = {"filename": name, "parentFileId": parent_id}
        if duplicate:
            payload["NotReuse"] = True
            payload["duplicate"] = duplicate
        return self.upload_request(
            payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def fs_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_move(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        parent_id: int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """移动

        POST https://www.123pan.com/api/file/mod_pid

        :payload:
            - fileIdList: list[FileID]

                .. code:: python

                    FileID = {
                        "FileId": int | str
                    }

            - parentFileId: int | str = 0
            - event: str = "fileMove"
        """
        if isinstance(payload, (int, str)):
            payload = {"fileIdList": [{"FileId": payload}]}
        elif not isinstance(payload, dict):
            payload = {"fileIdList": [{"FileId": fid} for fid in payload]}
        payload = dict_to_lower_merge(payload, {"parentFileId": parent_id, "event": "fileMove"})
        return self.request(
            "file/mod_pid", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload # type: ignore
    def fs_rename(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_rename(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_rename(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """（单个）改名

        POST https://www.123pan.com/api/file/rename

        :payload:
            - FileId: int | str
            - fileName: str
            - driveId: int | str = 0
            - duplicate: 0 | 1 | 2 = 0 💡 处理同名：0: 提示/忽略 1: 保留两者 2: 替换
            - event: str = "fileRename"
        """
        payload = dict_to_lower_merge(payload, {
            "driveId": 0, 
            "duplicate": 0, 
            "event": "fileRename", 
        })
        return self.request(
            "file/rename", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload # type: ignore
    def fs_trash(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        event: str = "intoRecycle", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_trash(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        event: str = "intoRecycle", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_trash(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        event: str = "intoRecycle", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """操作回收站

        POST https://www.123pan.com/api/file/trash

        :payload:
            - fileTrashInfoList: list[File] 💡 信息可以取自 `P123Client.fs_info` 接口

                .. code:: python

                    File = { 
                        "FileId": int | str, 
                        ...
                    }

            - driveId: int = 0
            - event: str = "intoRecycle" 💡 事件类型

                - "intoRecycle": 移入回收站
                - "recycleRestore": 移出回收站

            - operation: bool = <default>
        """
        if isinstance(payload, (int, str)):
            payload = {"fileTrashInfoList": [{"FileId": payload}]}
        elif not isinstance(payload, dict):
            payload = {"fileTrashInfoList": [{"FileId": fid} for fid in payload]}
        payload = dict_to_lower_merge(payload, {"driveId": 0, "event": event})
        if payload.get("operation") is None:
            match payload["event"]:
                case "recycleRestore":
                    payload["operation"] = False
                case _:
                    payload["operation"] = True
        return self.request(
            "file/trash", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def fs_trash_clear(
        self, 
        payload: dict = {"event": "recycleClear"}, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def fs_trash_clear(
        self, 
        payload: dict = {"event": "recycleClear"}, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def fs_trash_clear(
        self, 
        payload: dict = {"event": "recycleClear"}, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """清空回收站

        POST https://www.123pan.com/api/file/trash_delete_all

        :payload:
            - event: str = "recycleClear"
        """
        payload.setdefault("event", "recycleClear")
        return self.request(
            "file/trash_delete_all", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def offline_task_delete(
        self, 
        payload: int | Iterable[int] | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def offline_task_delete(
        self, 
        payload: int | Iterable[int] | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def offline_task_delete(
        self, 
        payload: int | Iterable[int] | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """删除离线下载任务

        POST https://www.123pan.com/api/offline_download/task/delete

        :payload:
            - task_ids: list[int] 💡 任务 id 列表
            - status_arr: list[0|1|2] = [] 💡 状态列表：0:等待 1:运行 2:完成
        """
        if isinstance(payload, int):
            payload = {"task_ids": [payload], "status_arr": []}
        elif not isinstance(payload, dict):
            if not isinstance(payload, (list, tuple)):
                payload = tuple(payload)
            payload = {"task_ids": payload, "status_arr": []}
        return self.request(
            "offline_download/task/delete", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def offline_task_list(
        self, 
        payload: dict | int = 1, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def offline_task_list(
        self, 
        payload: dict | int = 1, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def offline_task_list(
        self, 
        payload: dict | int = 1, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """离线下载任务列表

        POST https://www.123pan.com/api/offline_download/task/list

        :payload:
            - current_page: int = 1
            - page_size: 100
            - status_arr: list[0|1|2] = [0, 1] 💡 状态列表：0:等待 1:运行 2:完成
        """
        if isinstance(payload, int):
            payload = {"current_page": payload, "page_size": 100, "status_arr": [0, 1]}
        else:
            payload = {"current_page": 1, "page_size": 100, "status_arr": [0, 1], **payload}
        return self.request(
            "offline_download/task/list", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def offline_task_resolve(
        self, 
        payload: str | Iterable[str] | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def offline_task_resolve(
        self, 
        payload: str | Iterable[str] | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def offline_task_resolve(
        self, 
        payload: str | Iterable[str] | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """解析下载链接

        POST https://www.123pan.com/api/offline_download/task/resolve

        :payload:
            - urls: str = <default> 💡 下载链接，多个用 "\n" 隔开（用于新建链接下载任务）
            - info_hash: str = <default> 💡 种子文件的 info_hash（用于新建BT任务）
        """
        if isinstance(payload, str):
            payload = {"urls": payload.strip("\n")}
        elif not isinstance(payload, dict):
            payload = {"urls": "\n".join(payload)}
        return self.request(
            "offline_download/task/resolve", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    # TODO: 支持接受一个 Iterable[dict | int]，int 视为 id （select_file 为 [0]），dict 视为 resolve 信息
    @overload
    def offline_task_submit(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def offline_task_submit(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def offline_task_submit(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """提交离线下载任务

        POST https://www.123pan.com/api/offline_download/task/submit

        :payload:
            - resource_list: list[Task] 💡 资源列表

                .. code:: python

                    File = {
                        "resource_id": int, 
                        "select_file": list[int] # 如果是链接下载，则传 [0]，如果BT下载，则传需要下载的文件在列表中的索引的列表
                    }

            - upload_dir: int 💡 保存到目录的 id
        """
        return self.request(
            "offline_download/task/submit", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def offline_task_upload_seed(
        self, 
        /, 
        file: Buffer | SupportsRead[Buffer] | Iterable[Buffer], 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def offline_task_upload_seed(
        self, 
        /, 
        file: Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer], 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def offline_task_upload_seed(
        self, 
        /, 
        file: Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer], 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """上传种子，以作解析

        POST https://www.123pan.com/api/offline_download/upload/seed
        """
        if async_:
            headers, request_kwargs["data"] = encode_multipart_data_async({}, {"upload-torrent": file}, file_suffix=".torrent")
        else:
            headers, request_kwargs["data"] = encode_multipart_data({}, {"upload-torrent": file}, file_suffix=".torrent") # type: ignore
        request_kwargs["headers"] = {**(request_kwargs.get("headers") or {}), **headers}
        return self.request(
            "offline_download/upload/seed", 
            "POST", 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_cancel(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_cancel(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_cancel(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """取消分享

        POST https://www.123pan.com/api/share/delete

        :payload:
            - shareInfoList: list[ShareID] 💡 信息可以取自 `P123Client.fs_info` 接口

                .. code:: python

                    ShareID = { 
                        "shareId": int | str, 
                    }

            - driveId: int = 0
            - event: str = "shareCancel" 💡 事件类型
            - isPayShare: bool = False 💡 是否付费分享
        """
        if isinstance(payload, (int, str)):
            payload = {"shareInfoList": [{"shareId": payload}]}
        elif not isinstance(payload, dict):
            payload = {"shareInfoList": [{"shareId": sid} for sid in payload]}
        payload = dict_to_lower_merge(payload, {
            "driveId": 0, 
            "event": "shareCancel", 
            "isPayShare": False, 
        })
        return self.request(
            "share/delete", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_clear(
        self, 
        payload: dict = {"event": "shareClear"}, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_clear(
        self, 
        payload: dict = {"event": "shareClear"}, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_clear(
        self, 
        payload: dict = {"event": "shareClear"}, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """清理全部失效链接

        GET https://www.123pan.com/api/share/clean_expire

        :payload:
            - event: str = "shareClear"
        """
        return self.request(
            "share/clean_expire", 
            params=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_create(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_create(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_create(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """创建分享

        POST https://www.123pan.com/api/share/create

        :payload:
            - fileIdList: int | str 💡 文件或目录的 id，多个用逗号 "," 分隔
            - displayStatus: int = 2     💡 默认展示：1:平铺 2:列表
            - driveId: int = 0
            - event: str = "shareCreate" 💡 事件类型
            - expiration: "9999-12-31T23:59:59+08:00" 💡 有效期，日期用 ISO 格式
            - isPayShare: bool = False   💡 是否付费分享
            - isReward: 0 | 1 = 0        💡 是否开启打赏
            - payAmount: int = 0         💡 付费金额，单位：分
            - renameVisible: bool = False
            - resourceDesc: str = ""     💡 资源描述
            - shareName: str = <default> 💡 分享名称
            - sharePwd: str = ""         💡 分享密码
            - trafficLimit: int = 0      💡 流量限制额度，单位字节
            - trafficLimitSwitch: 1 | 2 = 1 💡 是否开启流量限制：1:关闭 2:开启
            - trafficSwitch: 1 | 2 = 1      💡 是否开启免登录流量包：1:关闭 2:开启
        """
        if isinstance(payload, (int, str)):
            payload = {"fileIdList": payload}
        elif not isinstance(payload, dict):
            payload = {"fileIdList": ",".join(map(str, payload))}
        payload = dict_to_lower_merge(payload, {
            "displayStatus": 2, 
            "driveId": 0, 
            "event": "shareCreate", 
            "expiration": "9999-12-31T23:59:59+08:00", 
            "isPayShare": False, 
            "isReward": 0, 
            "payAmount": 0, 
            "renameVisible": False, 
            "resourceDesc": "", 
            "sharePwd": "", 
            "trafficLimit": 0, 
            "trafficLimitSwitch": 1, 
            "trafficSwitch": 1, 
        })
        if "fileidlist" not in payload:
            raise ValueError("missing field: 'fileIdList'")
        if "sharename" not in payload:
            payload["sharename"] = "%d 个文件或目录" % (str(payload["fileidlist"]).count(",") + 1)
        return self.request(
            "share/create", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_download_info(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_download_info(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_download_info(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取分享中的下载信息

        POST https://www.123pan.com/api/share/download/info

        .. note::
            可以作为 staticmethod 使用，此时第 1 个位置参数要传入 None 或者 dict

            如果文件在 100MB 以内，下载时是不需要登录的；如果超过 100 MB，但分享者设置的免登录流量包未告罄，下载时也不需要登录

            你也可以使用 `P123Client.download_info` 来获取下载链接，则不需要提供 "ShareKey" 和 "SharePwd"

        :payload:
            - ShareKey: str 💡 分享码
            - SharePwd: str = <default> 💡 密码，如果没有就不用传
            - Etag: str
            - S3KeyFlag: str
            - FileID: int | str
            - Size: int = <default>
            - ...
        """
        if isinstance(self, dict):
            payload = self
            self = None
        assert payload is not None
        update_headers_in_kwargs(request_kwargs, platform="android")
        api = complete_url("share/download/info", base_url)
        if self is None:
            request_kwargs.setdefault("parse", default_parse)
            request = request_kwargs.pop("request", None)
            if request is None:
                request = get_default_request()
                request_kwargs["async_"] = async_
            return request(url=api, method="POST", json=payload, **request_kwargs)
        else:
            return self.request(
                api, 
                "POST", 
                json=payload, 
                async_=async_, 
                **request_kwargs, 
            )

    @overload
    def share_download_info_batch(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_download_info_batch(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_download_info_batch(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取分享中的批量下载信息

        POST https://www.123pan.com/api/file/batch_download_share_info

        .. note::
            可以作为 staticmethod 使用，此时第 1 个位置参数要传入 None 或者 dict

        :payload:
            - ShareKey: str 💡 分享码
            - SharePwd: str = <default> 💡 密码，如果没有就不用传
            - fileIdList: list[FileID]

                .. code:: python

                    FileID = {
                        "FileId": int | str
                    }
        """
        if isinstance(self, dict):
            payload = self
            self = None
        assert payload is not None
        api = complete_url("file/batch_download_share_info", base_url)
        if self is None:
            request_kwargs.setdefault("parse", default_parse)
            request = request_kwargs.pop("request", None)
            if request is None:
                request = get_default_request()
                request_kwargs["async_"] = async_
            return request(url=api, method="POST", json=payload, **request_kwargs)
        else:
            return self.request(
                api, 
                "POST", 
                json=payload, 
                async_=async_, 
                **request_kwargs, 
            )

    @overload
    def share_fs_copy(
        self, 
        payload: dict, 
        /, 
        parent_id: None | int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_fs_copy(
        self, 
        payload: dict, 
        /, 
        parent_id: None | int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_fs_copy(
        self, 
        payload: dict, 
        /, 
        parent_id: None | int | str = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """转存

        POST https://www.123pan.com/api/file/copy/async

        .. caution::
            这个函数的字段名，使用 snake case，而不是 camel case

        :payload:
            - share_key: str 💡 分享码
            - share_pwd: str = <default> 💡 密码，如果没有就不用传
            - current_level: int = 1
            - event: str = "transfer"
            - file_list: list[File]

                .. code:: python

                    File = {
                        "file_id": int | str, 
                        "file_name": str, 
                        "etag": str, 
                        "parent_file_id": int | str = 0, 
                        "drive_id": int | str = 0, 
                        ...
                    }
        """
        def to_snake_case(
            payload: dict[str, Any], 
            /, 
            *, 
            _map = {
                "sharekey": "share_key", 
                "sharepwd": "share_pwd", 
                "filelist": "file_list", 
                "fileid": "file_id", 
                "filename": "file_name", 
                "parentfileid": "parent_file_id", 
                "driveid": "drive_id", 
                "currentlevel": "current_level", 
            }.get, 
            _sub = re_compile("(?<!^)[A-Z]").sub, 
        ):
            d: dict[str, Any] = {}
            for k, v in payload.items():
                if "_" in k:
                    d[k.lower()] = v
                elif k2 := _map(k.lower()):
                    d[k2] = v
                elif (k2 := _sub(r"_\g<0>", k)) != k:
                    d[k2.lower()] = v
                else:
                    d[k] = v
            if "file_list" in d:
                ls = d["file_list"]
                for i, d2 in enumerate(ls):
                    ls[i] = {"drive_id": 0, **to_snake_case(d2)}
                    if parent_id is not None:
                        ls[i]["parent_file_id"] = parent_id
            return d
        payload = {"current_level": 1, "event": "transfer", **to_snake_case(payload)}
        return self.request(
            "file/copy/async", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_fs_list(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        request: None | Callable = None, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_fs_list(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        request: None | Callable = None, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_fs_list(
        self: None | dict | P123Client = None, 
        payload: None | dict = None, 
        /, 
        request: None | Callable = None, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取分享中的文件列表

        GET https://www.123pan.com/api/share/get

        .. note::
            如果返回信息中，"Next" 字段的值为 "-1"，代表最后一页（无需再翻页查询）

        :payload:
            - ShareKey: str 💡 分享码
            - SharePwd: str = <default> 💡 密码，如果没有就不用传
            - limit: int = 100 💡 分页大小，最多 100 个
            - next: int = 0    💡 下一批拉取开始的 id
            - orderBy: str = "file_name" 💡 排序依据

                - "file_name": 文件名
                - "size":  文件大小
                - "create_at": 创建时间
                - "update_at": 更新时间
                - ...

            - orderDirection: "asc" | "desc" = "asc" 💡 排序顺序
            - Page: int = 1 💡 第几页，从 1 开始，可以是 0
            - parentFileId: int | str = 0 💡 父目录 id
            - event: str = "homeListFile" 💡 事件名称
            - operateType: int | str = <default> 💡 操作类型
        """
        if isinstance(self, dict):
            payload = self
            self = None
        assert payload is not None
        payload = dict_to_lower_merge(cast(dict, payload), {
            "limit": 100, 
            "next": 0, 
            "orderBy": "file_name", 
            "orderDirection": "asc", 
            "Page": 1, 
            "parentFileId": 0, 
            "event": "homeListFile", 
        })
        request_kwargs.setdefault("parse", default_parse)
        api = complete_url("share/get", base_url)
        if self is None:
            if request is None:
                request = get_default_request()
                request_kwargs["async_"] = async_
            return request(url=api, method="GET", params=payload, **request_kwargs)
        else:
            return self.request(
                api, 
                params=payload, 
                async_=async_, 
                **request_kwargs, 
            )

    @overload # type: ignore
    def share_list(
        self, 
        payload: dict | int = 1, 
        /, 
        event: str = "shareListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_list(
        self, 
        payload: dict | int = 1, 
        /, 
        event: str = "shareListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_list(
        self, 
        payload: dict | int = 1, 
        /, 
        event: str = "shareListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取免费分享列表（可搜索）

        GET https://www.123pan.com/api/share/list

        .. note::
            如果返回信息中，"Next" 字段的值为 "-1"，代表最后一页（无需再翻页查询）

        :payload:
            - driveId: int | str = 0
            - limit: int = 100 💡 分页大小，最多 100 个
            - next: int = 0    💡 下一批拉取开始的 id
            - orderBy: str = "fileId" 💡 排序依据："fileId", ...
            - orderDirection: "asc" | "desc" = "desc" 💡 排序顺序
            - Page: int = <default> 💡 第几页，从 1 开始，可以是 0
            - event: str = "shareListFile"
            - operateType: int | str = <default>
            - SearchData: str = <default> 💡 搜索关键字（将无视 `parentFileId` 参数）
        """
        if isinstance(payload, int):
            payload = {"Page": payload}
        payload = dict_to_lower_merge(payload, {
            "driveId": 0, 
            "limit": 100, 
            "next": 0, 
            "orderBy": "fileId", 
            "orderDirection": "desc", 
            "event": event, 
        })
        return self.request(
            "share/list", 
            params=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_payment_list(
        self, 
        payload: dict | int = 1, 
        /, 
        event: str = "shareListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_payment_list(
        self, 
        payload: dict | int = 1, 
        /, 
        event: str = "shareListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_payment_list(
        self, 
        payload: dict | int = 1, 
        /, 
        event: str = "shareListFile", 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """获取付费分享列表（可搜索）

        GET https://www.123pan.com/api/restful/goapi/v1/share/content/payment/list

        .. note::
            如果返回信息中，"Next" 字段的值为 "-1"，代表最后一页（无需再翻页查询）

        :payload:
            - driveId: int | str = 0
            - limit: int = 100 💡 分页大小，最多 100 个
            - next: int = 0    💡 下一批拉取开始的 id
            - orderBy: str = "fileId" 💡 排序依据："fileId", ...
            - orderDirection: "asc" | "desc" = "desc" 💡 排序顺序
            - Page: int = <default> 💡 第几页，从 1 开始，可以是 0
            - event: str = "shareListFile"
            - operateType: int | str = <default>
            - SearchData: str = <default> 💡 搜索关键字（将无视 `parentFileId` 参数）
        """
        if isinstance(payload, int):
            payload = {"Page": payload}
        payload = dict_to_lower_merge(payload, {
            "driveId": 0, 
            "limit": 100, 
            "next": 0, 
            "orderBy": "fileId", 
            "orderDirection": "desc", 
            "event": event, 
        })
        return self.request(
            "restful/goapi/v1/share/content/payment/list", 
            params=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_reward_set(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        is_reward: bool = False, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_reward_set(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        is_reward: bool = False, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_reward_set(
        self, 
        payload: dict | int | str | Iterable[int | str], 
        /, 
        is_reward: bool = False, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """开启或关闭打赏

        POST https://www.123pan.com/api/restful/goapi/v1/share/reward/status

        :payload:
            - ids: list[int | str] 💡 分享 id
            - isReward: 0 | 1 = 1
        """
        if isinstance(payload, (int, str)):
            payload = {"ids": [payload]}
        elif not isinstance(payload, dict):
            payload = {"ids": list(payload)}
        payload = dict_to_lower_merge(payload, is_reward=int(is_reward))
        return self.request(
            "restful/goapi/v1/share/reward/status", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def share_traffic_set(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def share_traffic_set(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def share_traffic_set(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """流量包设置

        PUT https://www.123pan.com/api/restful/goapi/v1/share/info

        :payload:
            - shareId: int | str
            - trafficLimit: int = <default>         💡 流量限制额度，单位字节
            - trafficLimitSwitch: 1 | 2 = <default> 💡 是否开启流量限制：1:关闭 2:开启
            - trafficSwitch: 1 | 2 = <default>      💡 是否开启免登录流量包：1:关闭 2:开启
            - ...
        """
        return self.request(
            "restful/goapi/v1/share/info", 
            "PUT", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def upload_auth(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_auth(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_auth(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """认证上传信息，获取上传链接

        POST https://www.123pan.com/api/file/s3_upload_object/auth

        .. note::
            只能获取 1 个上传链接，用于非分块上传

        :payload:
            - bucket: str
            - key: str
            - storageNode: str
            - uploadId: str
        """
        return self.request(
            "file/s3_upload_object/auth", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload # type: ignore
    def upload_complete(
        self, 
        payload: dict, 
        /, 
        is_multipart: bool = False, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_complete(
        self, 
        payload: dict, 
        /, 
        is_multipart: bool = False, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_complete(
        self, 
        payload: dict, 
        /, 
        is_multipart: bool = False, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """完成上传

        POST https://www.123pan.com/api/file/upload_complete/v2

        :payload:
            - FileId: int 💡 文件 id
            - bucket: str 💡 存储桶
            - key: str
            - storageNode: str
            - uploadId: str
            - isMultipart: bool = True 💡 是否分块上传
        """
        payload = dict_to_lower_merge(payload, isMultipart=is_multipart)
        return self.request(
            "file/upload_complete/v2", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def upload_prepare(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_prepare(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_prepare(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """认证上传信息，获取上传链接

        POST https://www.123pan.com/api/file/s3_repare_upload_parts_batch

        .. note::
            一次可获取 `partNumberEnd - partNumberStart` 个上传链接，用于分块上传

        :payload:
            - bucket: str
            - key: str
            - storageNode: str
            - uploadId: str
            - partNumberStart: int = 1 💡 开始的分块编号（从 0 开始编号）
            - partNumberEnd: int = <default> 💡 结束的分块编号（不含）
        """
        if "partNumberStart" not in payload:
            payload["partNumberStart"] = 1
        if "partNumberEnd" not in payload:
            payload["partNumberEnd"] = int(payload["partNumberStart"]) + 1
        return self.request(
            "file/s3_repare_upload_parts_batch", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload # type: ignore
    def upload_list(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_list(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_list(
        self, 
        payload: dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """罗列已经上传的分块

        POST https://www.123pan.com/api/file/s3_list_upload_parts

        :payload:
            - bucket: str
            - key: str
            - storageNode: str
            - uploadId: str
        """
        return self.request(
            "file/s3_list_upload_parts", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    def upload_request(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_request(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_request(
        self, 
        payload: str | dict, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """请求上传，获取一些初始化信息

        POST https://www.123pan.com/api/file/upload_request

        .. note::
            当响应信息里面有 "Reuse" 的值为 "true"，说明已经存在目录或者文件秒传

        :payload:
            - fileName: str 💡 文件或目录的名字
            - driveId: int | str = 0
            - duplicate: 0 | 1 | 2 = 0 💡 处理同名：0: 提示/忽略 1: 保留两者 2: 替换
            - etag: str = "" 💡 文件的 MD5 散列值
            - parentFileId: int | str = 0 💡 父目录 id
            - size: int = 0 💡 文件大小，单位：字节
            - type: 0 | 1 = 1 💡 类型，如果是目录则是 1，如果是文件则是 0
            - NotReuse: bool = False 💡 不要重用（仅在 `type=1` 时有效，如果为 False，当有重名时，立即返回，此时 `duplicate` 字段无效）
            - ...
        """
        if isinstance(payload, str):
            payload = {"fileName": payload}
        payload = dict_to_lower_merge(payload, {
            "driveId": 0, 
            "duplicate": 0, 
            "etag": "", 
            "parentFileId": 0,
            "size": 0, 
            "type": 1, 
            "NotReuse": False, 
        })
        if payload["size"] or payload["etag"]:
            payload["type"] = 0
        return self.request(
            "file/upload_request", 
            "POST", 
            json=payload, 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    # TODO: 支持断点续传，也就是传入复传信息
    # TODO: 支持如果文件未曾打开，则可等尝试秒传失败之后，再行打开（因为如果能秒传，则根本不必打开）
    @overload # type: ignore
    def upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_file(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ), 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """上传文件

        .. note::
            如果文件名中包含字符 "\\/:*?|><，则转换为对应的全角字符

        :param file: 待上传的文件

            - 如果为 `collections.abc.Buffer`，则作为二进制数据上传
            - 如果为 `filewrap.SupportsRead`，则作为可读的二进制文件上传
            - 如果为 `str` 或 `os.PathLike`，则视为路径，打开后作为文件上传
            - 如果为 `yarl.URL` 或 `http_request.SupportsGeturl` (`pip install python-http_request`)，则视为超链接，打开后作为文件上传
            - 如果为 `collections.abc.Iterable[collections.abc.Buffer]` 或 `collections.abc.AsyncIterable[collections.abc.Buffer]`，则迭代以获取二进制数据，逐步上传

        :param file_md5: 文件的 MD5 散列值
        :param file_name: 文件名
        :param file_size: 文件大小
        :param parent_id: 要上传的目标目录
        :param duplicate: 处理同名：0: 提示/忽略 1: 保留两者 2: 替换
        :param async_: 是否异步
        :param request_kwargs: 其它请求参数

        :return: 接口响应
        """ 
        def gen_step():
            nonlocal file, file_md5, file_name, file_size
            def do_upload(file):
                return self.upload_file(
                    file=file, 
                    file_md5=file_md5, 
                    file_name=file_name, 
                    file_size=file_size, 
                    parent_id=parent_id, 
                    duplicate=duplicate, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
            try:
                file = getattr(file, "getbuffer")()
            except (AttributeError, TypeError):
                pass
            if isinstance(file, Buffer):
                file_size = buffer_length(file)
                if not file_md5:
                    file_md5 = md5(file).hexdigest()
            elif isinstance(file, (str, PathLike)):
                path = fsdecode(file)
                if not file_name:
                    file_name = basename(path)
                if async_:
                    async def request():
                        async with async_open(path, "rb") as file:
                            setattr(file, "fileno", file.file.fileno)
                            setattr(file, "seekable", lambda: True)
                            return await do_upload(file)
                    return request
                else:
                    return do_upload(open(path, "rb"))
            elif isinstance(file, SupportsRead):
                seek = getattr(file, "seek", None)
                seekable = False
                curpos = 0
                if callable(seek):
                    if async_:
                        seek = ensure_async(seek, threaded=True)
                    try:
                        seekable = getattr(file, "seekable")()
                    except (AttributeError, TypeError):
                        try:
                            curpos = yield seek(0, 1)
                            seekable = True
                        except Exception:
                            seekable = False
                if not file_md5:
                    if not seekable:
                        fsrc = file
                        file = TemporaryFile()
                        if async_:
                            yield copyfileobj_async(fsrc, file)
                        else:
                            copyfileobj(fsrc, file)
                        file.seek(0)
                        return do_upload(file)
                    try:
                        if async_:
                            file_size, hashobj = yield file_digest_async(file)
                        else:
                            file_size, hashobj = file_digest(file)
                    finally:
                        yield cast(Callable, seek)(curpos)
                    file_md5 = hashobj.hexdigest()
                if file_size < 0:
                    try:
                        fileno = getattr(file, "fileno")()
                        file_size = fstat(fileno).st_size - curpos
                    except (AttributeError, TypeError, OSError):
                        try:
                            file_size = len(file) - curpos # type: ignore
                        except TypeError:
                            if seekable:
                                try:
                                    file_size = (yield cast(Callable, seek)(0, 2)) - curpos
                                finally:
                                    yield cast(Callable, seek)(curpos)
            elif isinstance(file, (URL, SupportsGeturl)):
                if isinstance(file, URL):
                    url = str(file)
                else:
                    url = file.geturl()
                if async_:
                    from httpfile import AsyncHttpxFileReader
                    async def request():
                        file = await AsyncHttpxFileReader.new(url)
                        async with file:
                            return await do_upload(file)
                    return request
                else:
                    from httpfile import HTTPFileReader
                    with HTTPFileReader(url) as file:
                        return do_upload(file)
            elif not file_md5 or file_size < 0:
                if async_:
                    file = bytes_iter_to_async_reader(file) # type: ignore
                else:
                    file = bytes_iter_to_reader(file) # type: ignore
                return do_upload(file)
            if not file_name:
                file_name = getattr(file, "name", "")
                file_name = basename(file_name)
            if file_name:
                file_name = escape_filename(file_name)
            else:
                file_name = str(uuid4())
            if file_size < 0:
                file_size = getattr(file, "length", 0)
            resp = yield self.upload_request(
                {
                    "etag": file_md5, 
                    "fileName": file_name, 
                    "size": file_size, 
                    "parentFileId": parent_id, 
                    "type": 0, 
                    "duplicate": duplicate, 
                }, 
                base_url=base_url, 
                async_=async_, 
                **request_kwargs, 
            )
            if resp.get("code", 0) not in (0, 200):
                return resp
            upload_data = resp["data"]
            if upload_data["Reuse"]:
                return resp
            slice_size = int(upload_data["SliceSize"])
            upload_request_kwargs = {
                **request_kwargs, 
                "method": "PUT", 
                "headers": {"authorization": ""}, 
                "parse": ..., 
            }
            if file_size > slice_size:
                if async_:
                    async def request():
                        chunks = bio_chunk_async_iter(file, chunksize=slice_size) # type: ignore
                        upload_data["partNumberStart"] = 1
                        async for chunk in chunks:
                            resp = await self.upload_prepare(
                                upload_data, 
                                base_url=base_url, 
                                async_=True, 
                                **request_kwargs, 
                            )
                            check_response(resp)
                            await self.request(
                                resp["data"]["presignedUrls"]["1"], 
                                data=chunk, 
                                async_=True, 
                                **upload_request_kwargs, 
                            )
                            upload_data["partNumberStart"] += 1
                    yield request()
                else:
                    chunks = bio_chunk_iter(file, chunksize=slice_size) # type: ignore
                    for slice_no, chunk in enumerate(chunks, 1):
                        upload_data["partNumberStart"] = slice_no
                        resp = self.upload_prepare(
                            upload_data, 
                            base_url=base_url, 
                            **request_kwargs, 
                        )
                        check_response(resp)
                        self.request(
                            resp["data"]["presignedUrls"]["1"], 
                            data=chunk, 
                            **upload_request_kwargs, 
                        )
            else:
                resp = yield self.upload_auth(
                    upload_data, 
                    base_url=base_url, 
                    async_=async_, 
                    **request_kwargs, 
                )
                check_response(resp)
                yield self.request(
                    resp["data"]["presignedUrls"]["1"], 
                    data=file, 
                    async_=async_, 
                    **upload_request_kwargs, 
                )
            upload_data["isMultipart"] = file_size > slice_size
            return self.upload_complete(
                upload_data, 
                base_url=base_url, 
                async_=async_, 
                **request_kwargs, 
            )
        return run_gen_step(gen_step, async_=async_)

    @overload
    def upload_file_fast(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] ) = b"", 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def upload_file_fast(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ) = b"", 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def upload_file_fast(
        self, 
        /, 
        file: ( str | PathLike | URL | SupportsGeturl | 
                Buffer | SupportsRead[Buffer] | Iterable[Buffer] | AsyncIterable[Buffer] ) = b"", 
        file_md5: str = "", 
        file_name: str = "", 
        file_size: int = -1, 
        parent_id: int | str = 0, 
        duplicate: Literal[0, 1, 2] = 0, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """尝试秒传文件，如果失败也直接返回

        :param file: 待上传的文件

            - 如果为 `collections.abc.Buffer`，则作为二进制数据上传
            - 如果为 `filewrap.SupportsRead`，则作为可读的二进制文件上传
            - 如果为 `str` 或 `os.PathLike`，则视为路径，打开后作为文件上传
            - 如果为 `yarl.URL` 或 `http_request.SupportsGeturl` (`pip install python-http_request`)，则视为超链接，打开后作为文件上传
            - 如果为 `collections.abc.Iterable[collections.abc.Buffer]` 或 `collections.abc.AsyncIterable[collections.abc.Buffer]`，则迭代以获取二进制数据，逐步上传

        :param file_md5: 文件的 MD5 散列值
        :param file_name: 文件名
        :param file_size: 文件大小
        :param parent_id: 要上传的目标目录
        :param duplicate: 处理同名：0: 提示/忽略 1: 保留两者 2: 替换
        :param async_: 是否异步
        :param request_kwargs: 其它请求参数

        :return: 接口响应
        """ 
        def gen_step():
            nonlocal file, file_md5, file_name, file_size
            if file_md5 and file_size >= 0:
                pass
            elif file:
                def do_upload(file):
                    return self.upload_file_fast(
                        file=file, 
                        file_md5=file_md5, 
                        file_name=file_name, 
                        file_size=file_size, 
                        parent_id=parent_id, 
                        duplicate=duplicate, 
                        base_url=base_url, 
                        async_=async_, 
                        **request_kwargs, 
                    )
                try:
                    file = getattr(file, "getbuffer")()
                except (AttributeError, TypeError):
                    pass
                if isinstance(file, Buffer):
                    file_size = buffer_length(file)
                    if not file_md5:
                        file_md5 = md5(file).hexdigest()
                elif isinstance(file, (str, PathLike)):
                    path = fsdecode(file)
                    if not file_name:
                        file_name = basename(path)
                    if async_:
                        async def request():
                            async with async_open(path, "rb") as file:
                                return await do_upload(file)
                        return request
                    else:
                        return do_upload(open(path, "rb"))
                elif isinstance(file, SupportsRead):
                    if not file_md5 or file_size < 0:
                        if async_:
                            file_size, hashobj = yield file_digest_async(file)
                        else:
                            file_size, hashobj = file_digest(file)
                        file_md5 = hashobj.hexdigest()
                elif isinstance(file, (URL, SupportsGeturl)):
                    if isinstance(file, URL):
                        url = str(file)
                    else:
                        url = file.geturl()
                    if async_:
                        from httpfile import AsyncHttpxFileReader
                        async def request():
                            file = await AsyncHttpxFileReader.new(url)
                            async with file:
                                return await do_upload(file)
                        return request
                    else:
                        from httpfile import HTTPFileReader
                        with HTTPFileReader(url) as file:
                            return do_upload(file)
                elif not file_md5 or file_size < 0:
                    if async_:
                        file = bytes_iter_to_async_reader(file) # type: ignore
                    else:
                        file = bytes_iter_to_reader(file) # type: ignore
                    return do_upload(file)
            else:
                file_md5 = "d41d8cd98f00b204e9800998ecf8427e"
                file_size = 0
            if not file_name:
                file_name = getattr(file, "name", "")
                file_name = basename(file_name)
            if file_name:
                file_name = escape_filename(file_name)
            if not file_name:
                file_name = str(uuid4())
            if file_size < 0:
                file_size = getattr(file, "length", 0)
            return self.upload_request(
                {
                    "etag": file_md5, 
                    "fileName": file_name, 
                    "size": file_size, 
                    "parentFileId": parent_id, 
                    "type": 0, 
                    "duplicate": duplicate, 
                }, 
                base_url=base_url, 
                async_=async_, 
                **request_kwargs, 
            )
        return run_gen_step(gen_step, async_=async_)

    @overload
    def user_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    def user_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    def user_info(
        self, 
        /, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """用户信息

        GET https://www.123pan.com/api/user/info
        """
        return self.request(
            "user/info", 
            base_url=base_url, 
            async_=async_, 
            **request_kwargs, 
        )

    @overload
    @staticmethod
    def user_login(
        payload: dict, 
        /, 
        request: None | Callable = None, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False] = False, 
        **request_kwargs, 
    ) -> dict:
        ...
    @overload
    @staticmethod
    def user_login(
        payload: dict, 
        /, 
        request: None | Callable = None, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[True], 
        **request_kwargs, 
    ) -> Coroutine[Any, Any, dict]:
        ...
    @staticmethod
    def user_login(
        payload: dict, 
        /, 
        request: None | Callable = None, 
        base_url: str | Callable[[], str] = DEFAULT_BASE_URL, 
        *, 
        async_: Literal[False, True] = False, 
        **request_kwargs, 
    ) -> dict | Coroutine[Any, Any, dict]:
        """使用账号和密码登录

        POST https://www.123pan.com/api/user/sign_in

        .. note::
            获取的 token 有效期 30 天

        :payload:
            - passport: int | str   💡 手机号或邮箱
            - password: str         💡 密码
            - remember: bool = True 💡 是否记住密码（不用管）
        """
        api = complete_url("user/sign_in", base_url)
        request_kwargs.setdefault("parse", default_parse)
        if request is None:
            request = get_default_request()
            request_kwargs["async_"] = async_
        return request(url=api, method="POST", json=payload, **request_kwargs)

# TODO: 添加扫码登录接口，以及通过扫码登录的方法
